package de.ozgcloud.alfa.common;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

import org.apache.commons.lang3.StringUtils;

import de.itvsh.kop.common.datatype.StringBasedValue;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
class DeserializerHelper {

	public static String extractStringId(String url) {
		return URLDecoder.decode(url.substring(url.lastIndexOf('/') + 1), StandardCharsets.UTF_8);
	}

	public static Long extractLongId(String url) {
		var trimedUri = StringUtils.trimToNull(url);
		if (Objects.isNull(trimedUri)) {
			return null;
		}
		return Long.parseLong(extractStringId(trimedUri));
	}

	public static StringBasedValue extractStringBasedValue(Class<?> type, String url) {
		String value = DeserializerHelper.extractStringId(url);
		Method fromMethod;
		try {
			fromMethod = type.getMethod("from", String.class);
		} catch (NoSuchMethodException e) {
			throw new IllegalStateException(
					String.format("Cannot generate Id from type '%s'. Missing 'from' Method.", type.getSimpleName()));
		}
		try {
			return (StringBasedValue) fromMethod.invoke(null, value);
		} catch (IllegalAccessException | IllegalArgumentException | InvocationTargetException e) {
			throw new IllegalStateException(
					String.format("Cannot generate Id from type '%s'. Error calling 'from' Method.", type.getSimpleName()),
					e);
		}
	}
}