/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.apache.commons.lang3.reflect.ConstructorUtils;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.JsonDeserializer;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.deser.ContextualDeserializer;
import com.fasterxml.jackson.databind.deser.std.StdDeserializer;

import de.itvsh.kop.common.datatype.StringBasedValue;
import de.itvsh.kop.common.errorhandling.TechnicalException;

public class LinkedUserProfileResourceDeserializer extends StdDeserializer<Object> implements ContextualDeserializer {

	private static final long serialVersionUID = 1L;

	private BeanProperty beanProperty;
	private final JavaType targetType;
	private LinkedResource annotation;

	protected LinkedUserProfileResourceDeserializer() {
		super(Object.class);
		targetType = null;
	}

	protected LinkedUserProfileResourceDeserializer(BeanProperty beanProperty) {
		super(Object.class);
		this.beanProperty = beanProperty;
		this.targetType = beanProperty.getType();
		this.annotation = beanProperty.getAnnotation(LinkedResource.class);
	}

	@Override
	public Object deserialize(JsonParser jsonParser, DeserializationContext ctxt) throws IOException {
		if (jsonParser.isExpectedStartArrayToken()) {
			Collection<Object> idList = targetType.getRawClass().isAssignableFrom(Set.class) ? new HashSet<>() : new ArrayList<>();

			while (!jsonParser.nextToken().isStructEnd()) {
				idList.add(extractId(jsonParser.getText()));
			}
			return idList;
		} else {
			return extractId(jsonParser.getText());
		}
	}

	Object extractId(String url) {
		Class<?> type;
		if (targetType.isCollectionLikeType()) {
			type = targetType.getContentType().getRawClass();
		} else {
			type = targetType.getRawClass();
		}

		if (String.class.isAssignableFrom(type)) {
			return DeserializerHelper.extractStringId(url);
		}
		if (Long.class.isAssignableFrom(type) || Long.TYPE.isAssignableFrom(type)) {
			return DeserializerHelper.extractLongId(url);
		}
		if (StringBasedValue.class.isAssignableFrom(type)) {
			return DeserializerHelper.extractStringBasedValue(type, url);
		}
		return buildByBuilder(url);
	}

	private Object buildByBuilder(String url) {
		ObjectBuilder<?> builder;
		try {
			builder = ConstructorUtils.invokeConstructor(annotation.builder()).constructContextAware(beanProperty);
		} catch (NoSuchMethodException | IllegalAccessException | InvocationTargetException | InstantiationException e) {
			throw new TechnicalException("Error instanciating Builder.", e);
		}
		return builder.build(DeserializerHelper.extractStringId(url));
	}

	@Override
	public JsonDeserializer<?> createContextual(DeserializationContext ctxt, BeanProperty property) throws JsonMappingException {
		return new LinkedUserProfileResourceDeserializer(property);
	}
}