/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Objects;

import org.apache.commons.lang3.reflect.ConstructorUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.Link;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.ContextualSerializer;

import de.itvsh.kop.common.errorhandling.TechnicalException;
import de.ozgcloud.alfa.common.user.UserManagerUrlProvider;
import lombok.NoArgsConstructor;

@Component
@NoArgsConstructor
public class LinkedUserProfileResourceSerializer extends JsonSerializer<Object> implements ContextualSerializer {

	@Autowired
	private UserManagerUrlProvider userManagerUrlProvider;

	@Override
	public JsonSerializer<?> createContextual(SerializerProvider prov, BeanProperty property) {
		return new LinkedUserProfileResourceSerializer(property.getAnnotation(LinkedUserProfileResource.class), userManagerUrlProvider);
	}

	private LinkedUserProfileResource annotation;
	private UserManagerUrlProvider userManagerProvider;

	private LinkedUserProfileResourceSerializer(LinkedUserProfileResource annotation, UserManagerUrlProvider userManagerUrlProvider) {
		this.annotation = annotation;
		this.userManagerProvider = userManagerUrlProvider;
	}

	@Override
	public void serialize(Object value, JsonGenerator gen, SerializerProvider serializers) throws IOException {
		// FIXME Die null Abfrage gibt es nur, damit der UnitTest nicht auf die Bretter
		// geht
		if (Objects.nonNull(userManagerProvider) && userManagerProvider.isConfiguredForUserProfile()) {
			SerializerHelper.writeObject(gen, value, this::buildLink);
		} else {
			SerializerHelper.writeObject(gen, value);
		}
	}

	String buildLink(Object id) {
		return Link.of(buildUserProfileUri(id.toString())).getHref();
	}

	private String buildUserProfileUri(String id) {
		return String.format(userManagerProvider.getUserProfileTemplate(), getExtractor().extractId(id));
	}

	IdExtractor<Object> getExtractor() {
		try {
			return ConstructorUtils.invokeConstructor(annotation.extractor());
		} catch (NoSuchMethodException | IllegalAccessException | InvocationTargetException | InstantiationException e) {
			throw new TechnicalException("Error instanciating IdExtractor", e);
		}
	}
}