/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang;

import java.time.LocalDate;
import java.util.List;

import org.mapstruct.CollectionMappingStrategy;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.ReportingPolicy;

import de.itvsh.ozg.pluto.grpc.clientAttribute.GrpcClientAttribute;
import de.itvsh.ozg.pluto.grpc.clientAttribute.GrpcClientAttributeValue;
import de.itvsh.ozg.pluto.vorgang.GrpcVorgangHeader;
import de.ozgcloud.alfa.common.clientattribute.ClientAttributeService;
import de.ozgcloud.alfa.common.user.UserIdMapper;

@Mapper(uses = UserIdMapper.class, unmappedTargetPolicy = ReportingPolicy.WARN, collectionMappingStrategy = CollectionMappingStrategy.ADDER_PREFERRED)
interface VorgangHeaderMapper {

	static final String WIEDERVORLAGE_NEXT_FRIST_ATTRIBUTE_NAME = "nextWiedervorlageFrist";
	static final String HAS_POSTFACH_NACHRICHT_ATTRIBUTE_NAME = "hasPostfachNachricht";
	static final String HAS_NEW_POSTFACH_NACHRICHT_ATTRIBUTE_NAME = ClientAttributeService.HAS_NEW_POSTFACH_NACHRICHT_ATTRIBUTE_NAME;

	@Mapping(target = "nextFrist", source = "clientAttributesList")
	@Mapping(target = "hasPostfachNachricht", expression = "java(mapBoolClientAttributet(vorgangHeader.getClientAttributesList(), HAS_POSTFACH_NACHRICHT_ATTRIBUTE_NAME))")
	@Mapping(target = "hasNewPostfachNachricht", expression = "java(mapBoolClientAttributet(vorgangHeader.getClientAttributesList(), HAS_NEW_POSTFACH_NACHRICHT_ATTRIBUTE_NAME))")
	VorgangHeader toVorgangHeader(GrpcVorgangHeader vorgangHeader);

	default LocalDate mapNextFrist(List<GrpcClientAttribute> clientAttributes) {
		return ClientAttributeUtils.findByName(WIEDERVORLAGE_NEXT_FRIST_ATTRIBUTE_NAME, clientAttributes)
				.map(GrpcClientAttributeValue::getStringValue)
				.map(LocalDate::parse).findFirst().orElse(null);
	}

	default boolean mapBoolClientAttributet(List<GrpcClientAttribute> clientAttributes, String attributeName) {
		return ClientAttributeUtils.findByName(attributeName, clientAttributes)
				.map(GrpcClientAttributeValue::getBoolValue)
				.findFirst().orElse(false);
	}
}
