package de.ozgcloud.alfa.common;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.hateoas.EntityModel;
import org.springframework.test.util.ReflectionTestUtils;

import de.ozgcloud.alfa.common.user.UserManagerUrlProvider;

class ModelBuilderFactoryTest {

	@Spy
	@InjectMocks
	private ModelBuilderFactory<TestEntity> factory;
	@Mock
	private UserManagerUrlProvider userManagerUrlProvider;

	@Captor
	private ArgumentCaptor<ModelBuilder<TestEntity>> captor;

	@DisplayName("From entity")
	@Nested
	class TestFromEntity {

		@Test
		void shouldSetEntityToBuilder() {
			var entity = TestEntity.builder().build();

			var modelBuilder = factory.fromEntity(entity);

			verify(factory).withUserManagerUrl(captor.capture());
			assertThat(ReflectionTestUtils.getField(modelBuilder, "entity")).isEqualTo(entity);
		}
	}

	@DisplayName("From model")
	@Nested
	class TestFromModel {

		@Test
		void shouldSetModelToBuilder() {
			var entityModel = EntityModel.of(TestEntity.builder().build());

			var modelBuilder = factory.fromModel(entityModel);

			verify(factory).withUserManagerUrl(captor.capture());
			assertThat(ReflectionTestUtils.getField(modelBuilder, "model")).isEqualTo(entityModel);
		}
	}

	@DisplayName("WithUserManagerUrl")
	@Nested
	class TestWithUserManagerUrl {

		private final String userProfileUrl = "DummyUserProfileUrl";

		@Test
		void shouldHaveSetUserManagerUrl() {
			when(userManagerUrlProvider.getUserProfileTemplate()).thenReturn(userProfileUrl);

			var modelBuilder = factory.fromEntity(TestEntity.builder().build());

			var userManagerUrlValue = ReflectionTestUtils.getField(modelBuilder, "userManagerUrl");
			assertThat(userManagerUrlValue).isEqualTo(userProfileUrl);
		}
	}
}