/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.binaryfile;

import static org.mockito.Mockito.*;

import java.io.IOException;
import java.io.OutputStream;
import java.util.concurrent.CompletableFuture;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import com.google.protobuf.ByteString;

import de.itvsh.ozg.pluto.grpc.binaryFile.GrpcGetBinaryFileDataResponse;

class BinaryFileDownloadStreamObserverTest {

	@InjectMocks
	private BinaryFileDownloadStreamObserver downloadStreamObserver;
	@Mock
	private CompletableFuture<Boolean> streamFuture;
	@Mock
	private OutputStream out;

	@Nested
	class TestOnNext {

		@Mock
		private GrpcGetBinaryFileDataResponse response;

		@Test
		void shouldWriteToStreamOnReceivingContentPart() throws IOException {
			downloadStreamObserver.onNext(createFileConentResponse());

			verify(out).write(BinaryFileTestFactory.DATA);
		}

		private GrpcGetBinaryFileDataResponse createFileConentResponse() {
			return GrpcGetBinaryFileDataResponse.newBuilder().setFileContent(ByteString.copyFrom(BinaryFileTestFactory.DATA)).build();
		}
	}

	@Nested
	class TestOnError {

		@Mock
		private Throwable throwable;

		@Test
		void shouldCompleteFutureExceptionally() {
			downloadStreamObserver.onError(throwable);

			verify(streamFuture).completeExceptionally(throwable);
		}
	}

	@Nested
	class TestOnCompleted {

		@Test
		void shouldCompleteFuture() {
			downloadStreamObserver.onCompleted();

			verify(streamFuture).complete(true);
		}
	}
}