package de.ozgcloud.alfa.loeschanforderung;

import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.hateoas.EntityModel;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.common.errorhandling.ExceptionController;
import lombok.SneakyThrows;

class LoeschAnforderungControllerTest {

	@InjectMocks
	private LoeschAnforderungController controller;

	@Mock
	private LoeschAnforderungService loeschAnforderungService;

	@Mock
	private LoeschAnforderungModelAssembler modelAssembler;

	private MockMvc mockMvc;

	@BeforeEach
	void init() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).setControllerAdvice(new ExceptionController()).build();
	}

	@Nested
	class TestGetLoeschanforderung {

		private final LoeschAnforderung loeschAnforderung = LoeschAnforderungTestFactory.create();

		@BeforeEach
		void init() {
			when(loeschAnforderungService.getById(LoeschAnforderungTestFactory.ID)).thenReturn(loeschAnforderung);
			when(modelAssembler.toModel(loeschAnforderung)).thenReturn(EntityModel.of(loeschAnforderung));
		}

		@Test
		void shouldLoadLoeschAnforderung() {
			doRequest();

			verify(loeschAnforderungService).getById(LoeschAnforderungTestFactory.ID);
		}

		@Test
		void shouldCallModelAssembler() {
			doRequest();

			verify(modelAssembler).toModel(loeschAnforderung);
		}

		@Test
		void shouldReturn200() throws Exception {
			var response = doRequest();

			response.andExpect(status().isOk());
		}

		@SneakyThrows
		private ResultActions doRequest() {
			return mockMvc.perform(get(LoeschAnforderungController.BASE_PATH + "/" + LoeschAnforderungTestFactory.ID));
		}
	}
}