package de.ozgcloud.alfa.loeschanforderung;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.IanaLinkRelations;
import org.springframework.hateoas.Link;
import org.springframework.web.util.UriTemplate;

class LoeschAnforderungModelAssemblerTest {

	@InjectMocks
	private LoeschAnforderungModelAssembler modelAssembler;
	@Mock
	private LoeschAnforderungService loeschAnforderungService;

	@Nested
	@DisplayName("Build resource")
	class TestToModel {

		private static final UriTemplate LOESCH_ANFORDERUNG_COMMAND_URI_TEMPLATE = new UriTemplate(LoeschAnforderungCommandController.BASE_PATH);

		private final LoeschAnforderung loeschAnforderung = LoeschAnforderungTestFactory.create();

		@Test
		void shouldBuildModel() {
			var model = toModel();

			assertThat(model.getContent()).usingRecursiveComparison().isEqualTo(LoeschAnforderungTestFactory.create());
		}

		@Test
		void shouldHaveSelfLink() {
			var model = toModel();

			assertThat(model.getLink(IanaLinkRelations.SELF)).isPresent().get().extracting(Link::getHref)
					.isEqualTo(LoeschAnforderungController.BASE_PATH + "/" + LoeschAnforderungTestFactory.ID);
		}

		@Test
		void shouldCheckIfAllowedToVorgangLoeschen() {
			toModel();

			verify(loeschAnforderungService).isAllowedToVorgangLoeschen(loeschAnforderung);
		}

		@DisplayName("execute loeschen link")
		@Nested
		class TestExecuteLoeschenLink {

			@Test
			void shouldBeAdded() {
				when(loeschAnforderungService.isAllowedToVorgangLoeschen(any())).thenReturn(true);

				var model = toModel();

				assertThat(model.getLink(LoeschAnforderungModelAssembler.REL_EXECUTE_LOESCHEN)).isPresent().get().extracting(Link::getHref)
						.isEqualTo(LOESCH_ANFORDERUNG_COMMAND_URI_TEMPLATE.expand(LoeschAnforderungTestFactory.ID).toString());
			}

			@Test
			void shouldNotBeAdded() {
				when(loeschAnforderungService.isAllowedToVorgangLoeschen(any())).thenReturn(false);

				var model = toModel();

				assertThat(model.getLink(LoeschAnforderungModelAssembler.REL_EXECUTE_LOESCHEN)).isEmpty();
			}
		}

		@DisplayName("loeschAnforderung zuruecknehmen link")
		@Nested
		class TestLoeschAnforderungZuruecknehmen {

			@Test
			void shouldBeAdded() {
				var model = toModel();

				var link = model.getLink(LoeschAnforderungModelAssembler.REL_LOESCH_ANFORDERUNG_ZURUECKNEHMEN);
				assertThat(link).isPresent().map(Link::getHref)
						.hasValue(LOESCH_ANFORDERUNG_COMMAND_URI_TEMPLATE.expand(LoeschAnforderungTestFactory.ID).toString());
			}
		}

		private EntityModel<LoeschAnforderung> toModel() {
			return modelAssembler.toModel(loeschAnforderung);
		}
	}
}