package de.ozgcloud.alfa.statistic;

import static de.ozgcloud.alfa.statistic.StatisticRemoteService.*;
import static de.ozgcloud.alfa.statistic.StatisticService.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;

class StatisticServiceTest {

	@Spy
	@InjectMocks
	private StatisticService service;
	@Mock
	private StatisticRemoteService remoteService;
	@Mock
	private CurrentUserService currentUserService;

	@Nested
	class TestBuildGetVorgaengeStatisticResult {

		@BeforeEach
		void beforeEach() {
			when(remoteService.buildCountByStatusResultName(any())).thenCallRealMethod();
		}

		@Test
		void shouldContainCountWiedervorlage() {
			var statistic = service.buildGetVorgaengeStatisticResult(
					Map.of(COUNT_WIEDERVORLAGE_NEXT_FRIST_RESULT_NAME, StatisticResultTestFactory.create()));

			assertThat(statistic.getWiedervorlagen()).isEqualTo(StatisticResultTestFactory.INT_VALUE);
		}

		@Test
		void shouldContainDefaultCountWiedervorlage() {
			var statistic = service.buildGetVorgaengeStatisticResult(Map.of());

			assertThat(statistic.getWiedervorlagen()).isZero();
		}

		@Test
		void shouldContainExistsWiedervorlageOverdue() {
			var statistic = service.buildGetVorgaengeStatisticResult(Map.of(
					EXISTS_WIEDERVORLAGE_OVERDUE_RESULT_NAME,
					StatisticResultTestFactory.createBuilder().boolValue(true).build()));

			assertThat(statistic.isExistsWiedervorlageOverdue()).isTrue();
		}

		@Test
		void shouldContainDefaultExistsWiedervorlageOverdue() {
			var statistic = service.buildGetVorgaengeStatisticResult(Map.of());

			assertThat(statistic.isExistsWiedervorlageOverdue()).isFalse();
		}

		@Test
		void shouldContainCountByStatusNeu() {
			var statistic = service.buildGetVorgaengeStatisticResult(Map.of(
					String.format(COUNT_VORGANG_STATUS_RESULT_NAME_FORMAT, VorgangStatus.NEU.name()),
					StatisticResultTestFactory.create()));

			assertThat(statistic.getByStatus().getNeu()).isEqualTo(StatisticResultTestFactory.INT_VALUE);
		}

		@Test
		void shouldContainCountByStatusAngenommen() {
			var statistic = service.buildGetVorgaengeStatisticResult(Map.of(
					String.format(COUNT_VORGANG_STATUS_RESULT_NAME_FORMAT, VorgangStatus.ANGENOMMEN.name()),
					StatisticResultTestFactory.create()));

			assertThat(statistic.getByStatus().getAngenommen()).isEqualTo(StatisticResultTestFactory.INT_VALUE);
		}

		@Test
		void shouldContainCountByStatusInBearbeitung() {
			var statistic = service.buildGetVorgaengeStatisticResult(Map.of(
					String.format(COUNT_VORGANG_STATUS_RESULT_NAME_FORMAT, VorgangStatus.IN_BEARBEITUNG.name()),
					StatisticResultTestFactory.create()));

			assertThat(statistic.getByStatus().getInBearbeitung()).isEqualTo(StatisticResultTestFactory.INT_VALUE);
		}

		@Test
		void shouldContainCountByStatusBeschieden() {
			var statistic = service.buildGetVorgaengeStatisticResult(Map.of(
					String.format(COUNT_VORGANG_STATUS_RESULT_NAME_FORMAT, VorgangStatus.BESCHIEDEN.name()),
					StatisticResultTestFactory.create()));

			assertThat(statistic.getByStatus().getBeschieden()).isEqualTo(StatisticResultTestFactory.INT_VALUE);
		}

		@Test
		void shouldContainCountByStatusAbgeschlossen() {
			var statistic = service.buildGetVorgaengeStatisticResult(Map.of(
					String.format(COUNT_VORGANG_STATUS_RESULT_NAME_FORMAT, VorgangStatus.ABGESCHLOSSEN.name()),
					StatisticResultTestFactory.create()));

			assertThat(statistic.getByStatus().getAbgeschlossen()).isEqualTo(StatisticResultTestFactory.INT_VALUE);
		}

		@Test
		void shouldContainCountByStatusVerworfen() {
			var statistic = service.buildGetVorgaengeStatisticResult(Map.of(
					String.format(COUNT_VORGANG_STATUS_RESULT_NAME_FORMAT, VorgangStatus.VERWORFEN.name()),
					StatisticResultTestFactory.create()));

			assertThat(statistic.getByStatus().getVerworfen()).isEqualTo(StatisticResultTestFactory.INT_VALUE);
		}

		@Test
		void shouldContainCountByStatusZuLoeschen() {
			var statistic = service.buildGetVorgaengeStatisticResult(Map.of(
					String.format(COUNT_VORGANG_STATUS_RESULT_NAME_FORMAT, VorgangStatus.ZU_LOESCHEN.name()),
					StatisticResultTestFactory.create()));

			assertThat(statistic.getByStatus().getZuLoeschen()).isEqualTo(StatisticResultTestFactory.INT_VALUE);
		}

		@Test
		void shouldContainDefaultCountByStatus() {
			var statistic = service.buildGetVorgaengeStatisticResult(Map.of());

			assertThat(statistic.getByStatus().getNeu()).isZero();
		}
	}

	@Nested
	class TestGetCountByVorgangStatusList {

		@Test
		void shouldGetForVerwalungUser() {
			when(currentUserService.hasRole(UserRole.VERWALTUNG_USER)).thenReturn(true);

			var result = service.getCountByVorgangStatusList();

			assertThat(result).containsAll(VORGAENGE_STATUS_COUNT_VERWALTUNG_USER);
		}

		@Test
		void shouldGetForPoststelleUser() {
			when(currentUserService.hasRole(UserRole.VERWALTUNG_USER)).thenReturn(false);
			when(currentUserService.hasRole(UserRole.VERWALTUNG_POSTSTELLE)).thenReturn(true);

			var result = service.getCountByVorgangStatusList();

			assertThat(result).containsAll(VORGAENGE_STATUS_COUNT_POSTSTELLE_USER);
		}

		@Test
		void shouldGetForEinheitlicherAnsprechpartnerUser() {
			when(currentUserService.hasRole(UserRole.VERWALTUNG_USER)).thenReturn(false);
			when(currentUserService.hasRole(UserRole.VERWALTUNG_POSTSTELLE)).thenReturn(false);
			when(currentUserService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER)).thenReturn(true);

			var result = service.getCountByVorgangStatusList();

			assertThat(result).containsAll(VORGAENGE_STATUS_COUNT_EINHEITLICHER_ANSPRECHPARTNER);
		}

		@Test
		void shouldGetForUnknownUser() {
			when(currentUserService.hasRole(UserRole.VERWALTUNG_USER)).thenReturn(false);
			when(currentUserService.hasRole(UserRole.VERWALTUNG_POSTSTELLE)).thenReturn(false);
			when(currentUserService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER)).thenReturn(false);

			var result = service.getCountByVorgangStatusList();

			assertThat(result).containsAll(Collections.emptyList());
		}
	}

	@Nested
	class TestGetVorgaengeStatistics {

		private final List<VorgangStatus> countByVorgangStatus = List.of(VorgangStatus.ZU_LOESCHEN);

		@BeforeEach
		void beforeEach() {
			when(remoteService.buildCountByStatusResultName(any())).thenCallRealMethod();
		}

		@Test
		void shouldCallRemoteService() {
			service.getVorgaengeStatistic();

			verify(remoteService).getVorgaengeStatistics(countByVorgangStatus);
		}

		@Test
		void shouldBuildResult() {
			var response = Map.of(COUNT_WIEDERVORLAGE_NEXT_FRIST_RESULT_NAME, StatisticResultTestFactory.create());
			when(remoteService.getVorgaengeStatistics(countByVorgangStatus)).thenReturn(response);

			service.getVorgaengeStatistic();

			verify(service).buildGetVorgaengeStatisticResult(response);
		}

		@Test
		@Disabled("activate if new grpc api in use")
		void shouldCallGetCountByVorgangStatusList() {
			service.getVorgaengeStatistic();

			verify(service).getCountByVorgangStatusList();
		}
	}

	// TODO sebo: später löschen
	@DisplayName("Get statistic")
	@Nested
	class TestGetStatistic {

		private static final int WIEDERVORLAGE_COUNT = 8;
		private CountVorgangResponse response = CountVorgangResponseTestFactory.createBuilder()
				.pathCountResult(Map.of(StatisticService.COUNT_WIEDERVORLAGE_NEXT_FRIST_KEY, WIEDERVORLAGE_COUNT)).build();

		@Test
		void shouldCallRemoteService() {
			when(remoteService.countVorgang(any())).thenReturn(response);

			service.countVorgang();

			verify(remoteService).countVorgang(List.of(StatisticService.COUNT_WIEDERVORLAGE_NEXT_FRIST));
		}

		@Test
		void shouldBuildStatistic() {
			when(remoteService.countVorgang(any())).thenReturn(response);

			service.countVorgang();

			verify(service).buildStatistic(response);
		}

		@DisplayName("build statistic")
		@Nested
		class TestBuildStatistic {

			@Test
			void shouldSetByStatus() {
				var statistic = service.buildStatistic(response);

				assertThat(statistic.getByStatus()).usingRecursiveComparison().isEqualTo(ByStatusTestFactory.create());
			}

			@Test
			void shouldSetWiedervorlagen() {
				var statistic = service.buildStatistic(response);

				assertThat(statistic.getWiedervorlagen()).isEqualTo(WIEDERVORLAGE_COUNT);
			}

			@DisplayName("extract wiedervorlage")
			@Nested
			class TestExtractWiedervorlage {

				@Test
				void shouldReturnValueIfExists() {
					var count = 5;
					var countVorgangResponse = CountVorgangResponseTestFactory.createBuilder()
							.pathCountResult(Map.of(StatisticService.COUNT_WIEDERVORLAGE_NEXT_FRIST_KEY, count)).build();

					var wiedervorlageCount = service.extractWiedervorlageCount(countVorgangResponse);

					assertThat(wiedervorlageCount).isEqualTo(count);
				}

				@Test
				void shouldReturnZeroIfNotExists() {
					var wiedervorlageCount = service.extractWiedervorlageCount(
							CountVorgangResponseTestFactory.createBuilder().pathCountResult(Collections.emptyMap()).build());

					assertThat(wiedervorlageCount).isZero();
				}
			}
		}
	}

}
