/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.itvsh.kop.pluto.common.grpc.GrpcFormDataMapper;
import de.itvsh.ozg.pluto.vorgang.GrpcEingang;

class EingangMapperTest {

	@InjectMocks
	private final EingangMapper mapper = Mappers.getMapper(EingangMapper.class);

	@Mock
	private AntragstellerMapper antragstellerMapper;
	@Mock
	private EingangHeaderMapper eingangHeaderMapper;
	@Spy
	private final GrpcFormDataMapper formDataMapper = Mappers.getMapper(GrpcFormDataMapper.class);

	@DisplayName("From grpc")
	@Nested
	class TestFromGrpc {

		@BeforeEach
		void mockMapperReturnValues() {
			lenient().when(antragstellerMapper.toAntragsteller(any())).thenReturn(AntragstellerTestFactory.create());
			lenient().when(eingangHeaderMapper.toEingangHeader(any())).thenReturn(EingangHeaderTestFactory.create());
		}

		@Test
		void shouldMapIdToId() {
			var eingang = fromGrpc();

			assertThat(eingang.getId()).isEqualTo(GrpcEingangTestFactory.ID);
		}

		@Test
		void shouldHaveAntragsteller() {
			var eingang = fromGrpc();

			assertThat(eingang.getAntragsteller()).usingRecursiveComparison().isEqualTo(AntragstellerTestFactory.create());
		}

		@Test
		void shouldHaveEingangHeader() {
			var eingang = fromGrpc();

			assertThat(eingang.getHeader()).usingRecursiveComparison().isEqualTo(EingangHeaderTestFactory.create());
		}

		private Eingang fromGrpc() {
			return fromGrpc(GrpcEingangTestFactory.create());
		}

		private Eingang fromGrpc(GrpcEingang eingang) {
			return mapper.fromGrpc(GrpcEingangTestFactory.create());
		}
	}

	@DisplayName("With")
	@Nested
	class TestWithEmpty {

		@DisplayName("empty eingang")
		@Nested
		class TestEingang {

			@Test
			void shouldProceedFromGrpcMapping() {
				var eingang = mapper.fromGrpc(GrpcEingang.newBuilder().build());

				assertThat(eingang.getAntragsteller()).isNull();
				assertThat(eingang.getHeader()).isNull();
			}
		}
	}
}