/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang;

import static de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory.*;

import de.itvsh.ozg.pluto.grpc.clientAttribute.GrpcClientAttributeValue;
import de.itvsh.ozg.pluto.vorgang.GrpcEingang;
import de.itvsh.ozg.pluto.vorgang.GrpcVorgangWithEingang;
import de.ozgcloud.alfa.common.clientattribute.ClientAttributeService;

class GrpcVorgangWithEingangTestFactory {

	static GrpcVorgangWithEingang create() {
		return createBuilder().build();
	}

	static GrpcVorgangWithEingang createWithEingang(GrpcEingang eingang) {
		return createBuilder().setEingang(eingang).build();
	}

	static GrpcVorgangWithEingang.Builder createBuilder() {
		return GrpcVorgangWithEingang.newBuilder()
				.setStatus(STATUS.name())
				.setNummer(NUMMER)
				.setAktenzeichen(AKTENZEICHEN)
				.setAssignedTo(ASSIGNED_TO)
				.setCreatedAt(CREATED_AT_STR)
				.clearClientAttributes()
				.addClientAttributes(
						GrpcClientAttributeTestFactory.createBuilder()
								.setAttributeName(ClientAttributeService.HAS_NEW_POSTFACH_NACHRICHT_ATTRIBUTE_NAME)
								.setValue(GrpcClientAttributeValue.newBuilder().setBoolValue(true)).build())
				.setEingang(GrpcEingangTestFactory.create());
	}
}