package de.ozgcloud.alfa.export;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;
import de.xoev.xdomea.FormatType;

class VersionTypeCreatorTest {

	@InjectMocks
	private VersionTypeCreator creator;

	@Mock
	private FormatTypeCreator formatTypeCreator;

	@Nested
	class TestCreate {

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();
		private final OzgFile ozgFile = OzgFileTestFactory.create();
		private final FormatType formatType = FormaTypeTestFactory.create();

		@BeforeEach
		void init() {
			when(formatTypeCreator.create(vorgang, ozgFile)).thenReturn(formatType);
		}

		@Test
		void shouldHaveNummer() {
			var version = creator.create(vorgang, ozgFile);

			assertThat(version.getNummer()).isEqualTo(VersionTypeCreator.NUMMER);
		}

		@Test
		void shouldCreateFormatType() {
			creator.create(vorgang, ozgFile);

			verify(formatTypeCreator).create(vorgang, ozgFile);
		}

		@Test
		void shouldHaveFormatType() {
			var version = creator.create(vorgang, ozgFile);

			assertThat(version.getFormat()).containsExactly(formatType);
		}
	}

}