/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { getTestElement } from '../../support/cypress-helper';

export class VorgangFormularDatenHistorieE2EComponent {

	private readonly locatorRoot: string = 'historie-in-vorgang-container';
	private readonly locatorCreatedAt: string = 'historie-vorgang-created-at';

	public getRoot() {
		return getTestElement(this.locatorRoot);
	}

	public getCreatedAt() {
		return getTestElement(this.locatorCreatedAt);
	}

	public getListItemByIndex(index: number): VorgangFormularDatenHistorieItemE2EComponent {
		return new VorgangFormularDatenHistorieItemE2EComponent(index);
	}
}

export class VorgangFormularDatenHistorieItemE2EComponent {

	private readonly locatorRoot: string = 'historie-list-item-';
	private readonly locatorHeadline: string = 'historie-item-header';
	private readonly locatorUser: string = 'user-profile-in-historie-item-header';
	private readonly locatorSystemUser: string = 'system-user-in-historie-item-header';
	private readonly locatorExpandButton: string = 'expansion-button';

	constructor(private itemIndex: number) {
		this.locatorRoot = 'historie-list-item-' + this.itemIndex;
	}

	public getRoot() {
		return getTestElement(this.locatorRoot);
	}

	public getHeadline() {
		return this.getRoot().findTestElementWithClass(this.locatorHeadline);
	}

	public getUser() {
		return this.getRoot().findTestElementWithClass(this.locatorUser);
	}

	public getSystemUser() {
		return this.getRoot().findTestElementWithClass(this.locatorSystemUser);
	}

	public getExpandButton() {
		return this.getRoot().findTestElementWithClass(this.locatorExpandButton);
	}

	public getPostfachNachricht(): PostfachNachrichtHistorieItemE2EComponent {
		return new PostfachNachrichtHistorieItemE2EComponent(this.locatorRoot);
	}

	public getWiedervorlage(): WiedervorlageHistorieItemE2EComponent {
		return new WiedervorlageHistorieItemE2EComponent(this.locatorRoot);
	}

	public getKommentar(): KommentarHistorieItemE2EComponent {
		return new KommentarHistorieItemE2EComponent(this.locatorRoot);
	}
}

export class PostfachNachrichtHistorieItemE2EComponent {

	private readonly locatorRoot: string;

	private readonly locatorPostfachMailBody: string = 'postfach-nachricht-mail-body';
	private readonly locatorPostfachSubject: string = 'postfach-nachricht-subject';

	constructor(rootLocator: string) {
		this.locatorRoot = rootLocator;
	}

	public getRoot() {
		return getTestElement(this.locatorRoot);
	}

	public getPostfachNachrichtSubject() {
		return this.getRoot().findTestElementWithClass(this.locatorPostfachSubject);
	}

	public getPostfachNachrichtMailBody() {
		return this.getRoot().findTestElementWithClass(this.locatorPostfachMailBody);
	}
}

export class WiedervorlageHistorieItemE2EComponent {

	private readonly locatorRoot: string;

	private readonly locatorStatus: string = 'wiedervorlage-status';
	private readonly locatorBetreff: string = 'wiedervorlage-betreff';
	private readonly locatorBeschreibung: string = 'wiedervorlage-beschreibung';
	private readonly locatorAttachment: string = 'historie-item-attachment';

	constructor(rootLocator: string) {
		this.locatorRoot = rootLocator;
	}

	public getRoot() {
		return getTestElement(this.locatorRoot);
	}

	public getStatus() {
		return this.getRoot().findTestElementWithClass(this.locatorStatus);
	}

	public getBetreff() {
		return this.getRoot().findTestElementWithClass(this.locatorBetreff);
	}

	public getBeschreibung() {
		return this.getRoot().findTestElementWithClass(this.locatorBeschreibung);
	}

	public getAttachment() {
		return this.getRoot().findTestElementWithClass(this.locatorAttachment);
	}
}

export class KommentarHistorieItemE2EComponent {

	private readonly locatorRoot: string;

	private readonly locatorText: string = 'kommentar-text';
	private readonly locatorAttachment: string = 'historie-item-attachment';

	constructor(rootLocator: string) {
		this.locatorRoot = rootLocator;
	}

	public getRoot() {
		return getTestElement(this.locatorRoot);
	}

	public getText() {
		return this.getRoot().findTestElementWithClass(this.locatorText);
	}

	public getAttachment() {
		return this.getRoot().findTestElementWithClass(this.locatorAttachment);
	}
}