/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CyHttpMessages } from 'cypress/types/net-stubbing';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { HttpMethodE2E } from '../../../model/util';
import { VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections, interceptWithResponse } from '../../../support/cypress-helper';
import { exist, notExist } from '../../../support/cypress.util';
import { LinkRelE2E } from '../../../support/linkrels';
import { removeLinkFromResource } from '../../../support/tech.util';
import { loginAsSabine } from '../../../support/user-util';
import { createVorgang, initVorgang } from '../../../support/vorgang-util';

describe('Postfach Nachrichten', () => {
	const vorgangList: VorgangListE2EComponent = new MainPage().getVorgangList();
	const vorgangPage: VorgangPage = new VorgangPage();

	const vorgang: VorgangE2E = createVorgang();

	before(() => {
		initVorgang(vorgang);

		loginAsSabine();

		waitForSpinnerToDisappear();
		exist(vorgangList.getRoot());
	})

	after(() => {
		dropCollections();
	})

	describe('navigate to vorganDetail', () => {

		beforeEach(() => {
			interceptWithResponse(HttpMethodE2E.GET, '*/vorgangs/*', (req) => modifyResponse(req));
		})

		function modifyResponse(req: CyHttpMessages.IncomingHttpRequest): void {
			req.reply(res => res.send(removeLinkFromResource(res.body, LinkRelE2E.POSTFACH_NACHRICHTEN)));
		}

		it('navigate to vorgangDetail', () => {
			vorgangList.getListItem(vorgang.name).getRoot().click();
			waitForSpinnerToDisappear();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		it('should hide postfach nachrichten area if no postfach is configured', () => {
			notExist(vorgangPage.getPostfachMailcontainer().getRoot());
		})
	})
})