/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { NavigationE2EComponent } from 'apps/goofy-e2e/src/components/navigation/navigation.e2e.component';
import { VorgangListE2EComponent } from 'apps/goofy-e2e/src/components/vorgang/vorgang-list.e2e.component';
import { VorgangViewsE2EComponent } from 'apps/goofy-e2e/src/components/vorgang/vorgang-views.e2e.component';
import { MainPage, waitForSpinnerToDisappear } from 'apps/goofy-e2e/src/page-objects/main.po';
import { exist } from 'apps/goofy-e2e/src/support/cypress.util';
import { Interception } from 'cypress/types/net-stubbing';
import { HttpMethodE2E } from '../../../model/util';
import { VorgangE2E, VorgangStatusE2E } from '../../../model/vorgang';
import { dropCollections, intercept, scrollToWindowBottom, waitOfInterceptor } from '../../../support/cypress-helper';
import { generateIds } from '../../../support/tech.util';
import { getUserManagerUserSabine, getUserSabineInternalId, initUsermanagerUsers, loginAsSabine } from '../../../support/user-util';
import { buildVorgang, initVorgaenge } from '../../../support/vorgang-util';

describe('VorgangList View Pages', () => {
	const mainPage: MainPage = new MainPage();

	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const navigation: NavigationE2EComponent = mainPage.getNavigation();

	const views: VorgangViewsE2EComponent = mainPage.getViews();

	const PAGE_SIZE: number = 100;
	const assignedTo: string = getUserSabineInternalId();

	before(() => {
		const vorgaenge: VorgangE2E[] =
			createVorgaengeWithStatus(VorgangStatusE2E.NEU).concat(
			createAssignedVorgaengeWithStatus(VorgangStatusE2E.NEU),
			createVorgaengeWithStatus(VorgangStatusE2E.ANGENOMMEN),
			createAssignedVorgaengeWithStatus(VorgangStatusE2E.ANGENOMMEN),
			createVorgaengeWithStatus(VorgangStatusE2E.IN_BEARBEITUNG),
			createAssignedVorgaengeWithStatus(VorgangStatusE2E.IN_BEARBEITUNG),
			createVorgaengeWithStatus(VorgangStatusE2E.BESCHIEDEN),
			createAssignedVorgaengeWithStatus(VorgangStatusE2E.BESCHIEDEN),
			createVorgaengeWithStatus(VorgangStatusE2E.ABGESCHLOSSEN),
			createAssignedVorgaengeWithStatus(VorgangStatusE2E.ABGESCHLOSSEN),
			createVorgaengeWithStatus(VorgangStatusE2E.VERWORFEN),
			createAssignedVorgaengeWithStatus(VorgangStatusE2E.VERWORFEN));
		initVorgaenge(vorgaenge);
		initUsermanagerUsers([getUserManagerUserSabine()]);

		loginAsSabine();
		exist(vorgangList.getRoot());
		waitForSpinnerToDisappear();
	})

	after(() => {
		dropCollections();
	})

	describe('Neu', () => {

		const ALLE_NAME_PREFIX: string = 'VorgangNEU_';
		const MEINE_VORGAENGE_NAME_PREFIX: string = 'VorgangNEUAssigned_';

		describe('Alle filter', () => {

			describe('page 1', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangNeuPageLink(1)).as(loadPageInterceptor);
				})

				it('should show page 1 with 100 vorgaenge NEU', () => {
					views.getNeu().getRoot().click();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, ALLE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, ALLE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 2', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangNeuPageLink(2)).as(loadPageInterceptor);
				})

				it('should show page 2 with 10 vorgaenge NEU and 90 vorgaenge ASSIGNED on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 3', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangNeuPageLink(3)).as(loadPageInterceptor);
				})

				it('should show page 3 with 10 vorgaenge NEU ASSIGNED on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === 20);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 80);
						assert(interception.response.body._embedded.vorgangHeaderList[19].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			function buildVorgangNeuPageLink(page: number): string {
				return `${buildVorgangPageLink(page)}&status=NEU`;
			}
		})

		describe('Meine Vorgänge filter', () => {

			describe('page 1', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangNeuAssignedPageLink(1)).as(loadPageInterceptor);
				})

				it('should show page 1 with 100 vorgaenge NEU', () => {
					navigation.getMeineVorgaengeFilter().click();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 2', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangNeuAssignedPageLink(2)).as(loadPageInterceptor);
				})

				it('should show page 2 with 10 vorgaenge NEU and 90 vorgaenge ASSIGNED on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === 10);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 101);
						assert(interception.response.body._embedded.vorgangHeaderList[9].name, MEINE_VORGAENGE_NAME_PREFIX + 110);
					})
				})
			})

			function buildVorgangNeuAssignedPageLink(page: number): string {
				return `${buildVorgangPageLink(page)}&assignedTo=${assignedTo}&status=NEU`;
			}
		})

		describe('Nicht zugewiesen filter', () => {

			describe('page 1', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangUnassignedPageLink(1)).as(loadPageInterceptor);
				})

				it('should show page 1 with 100 vorgaenge NEU', () => {
					navigation.getNichtZugewiesenFilter().click();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, ALLE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, ALLE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 2', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangUnassignedPageLink(2)).as(loadPageInterceptor);
				})

				it('should show page 2 with 10 vorgaenge on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === 10);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, ALLE_NAME_PREFIX + 101);
						assert(interception.response.body._embedded.vorgangHeaderList[9].name, ALLE_NAME_PREFIX + 110);
					})
				})
			})

			function buildVorgangUnassignedPageLink(page: number): string {
				return `${buildVorgangPageLink(page)}&assignedTo=&status=NEU`;
			}
		})
	})

	describe('Angenommen', () => {

		const ALLE_NAME_PREFIX: string = 'VorgangANGENOMMEN_';
		const MEINE_VORGAENGE_NAME_PREFIX: string = 'VorgangANGENOMMENAssigned_';

		describe('Alle filter', () => {

			describe('page 1', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					navigation.getAlleFilter().click();
					waitForSpinnerToDisappear();
					intercept(HttpMethodE2E.GET, buildVorgangAngenommenPageLink(1)).as(loadPageInterceptor);
				})

				it('should show page 1 with 100 vorgaenge ANGENOMMEN', () => {
					views.getAngenommen().getRoot().click();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, ALLE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, ALLE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 2', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangAngenommenPageLink(2)).as(loadPageInterceptor);
				})

				it('should show page 2 with 10 vorgaenge ANGENOMMEN and 90 vorgaenge ASSIGNED on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 3', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangAngenommenPageLink(3)).as(loadPageInterceptor);
				})

				it('should show page 3 with 10 vorgaenge ANGENOMMEN ASSIGNED on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === 20);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 80);
						assert(interception.response.body._embedded.vorgangHeaderList[19].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			function buildVorgangAngenommenPageLink(page: number): string {
				return `${buildVorgangPageLink(page)}&status=ANGENOMMEN`;
			}
		})

		describe('Meine Vorgänge filter', () => {

			describe('page 1', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangAngenommenAssignedPageLink(1)).as(loadPageInterceptor);
				})

				it('should show page 1 with 100 vorgaenge ANGENOMMEN', () => {
					navigation.getMeineVorgaengeFilter().click();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 2', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangAngenommenAssignedPageLink(2)).as(loadPageInterceptor);
				})

				it('should show page 2 with 10 vorgaenge ANGENOMMEN and 90 vorgaenge ASSIGNED on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === 10);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 101);
						assert(interception.response.body._embedded.vorgangHeaderList[9].name, MEINE_VORGAENGE_NAME_PREFIX + 110);
					})
				})
			})

			function buildVorgangAngenommenAssignedPageLink(page: number): string {
				return `${buildVorgangPageLink(page)}&assignedTo=${assignedTo}&status=ANGENOMMEN`;
			}
		})

		describe('Nicht zugewiesen filter', () => {

			describe('page 1', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangAngenommenUnassignedPageLink(1)).as(loadPageInterceptor);
				})

				it('should show page 1 with 100 vorgaenge ANGENOMMEN', () => {
					navigation.getNichtZugewiesenFilter().click();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, ALLE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, ALLE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 2', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangAngenommenUnassignedPageLink(2)).as(loadPageInterceptor);
				})

				it('should show page 2 with 10 vorgaenge on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === 10);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, ALLE_NAME_PREFIX + 101);
						assert(interception.response.body._embedded.vorgangHeaderList[9].name, ALLE_NAME_PREFIX + 110);
					})
				})
			})

			function buildVorgangAngenommenUnassignedPageLink(page: number): string {
				return `${buildVorgangPageLink(page)}&assignedTo=&status=ANGENOMMEN`;
			}
		})
	})

	describe('In Bearbeitung', () => {

		const ALLE_NAME_PREFIX: string = 'VorgangIN_BEARBEITUNG_';
		const MEINE_VORGAENGE_NAME_PREFIX: string = 'VorgangIN_BEARBEITUNGAssigned_';

		describe('Alle filter', () => {

			describe('page 1', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					navigation.getAlleFilter().click();
					waitForSpinnerToDisappear();
					intercept(HttpMethodE2E.GET, buildVorgangInBearbeitungPageLink(1)).as(loadPageInterceptor);
				})

				it('should show page 1 with 100 vorgaenge IN_BEARBEITUNG', () => {
					views.getInBearbeitung().getRoot().click();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, ALLE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, ALLE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 2', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangInBearbeitungPageLink(2)).as(loadPageInterceptor);
				})

				it('should show page 2 with 10 vorgaenge IN_BEARBEITUNG and 90 vorgaenge ASSIGNED on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 3', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangInBearbeitungPageLink(3)).as(loadPageInterceptor);
				})

				it('should show page 3 with 10 vorgaenge IN_BEARBEITUNG ASSIGNED on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === 20);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 80);
						assert(interception.response.body._embedded.vorgangHeaderList[19].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			function buildVorgangInBearbeitungPageLink(page: number): string {
				return `${buildVorgangPageLink(page)}&status=IN_BEARBEITUNG`;
			}
		})

		describe('Meine Vorgänge filter', () => {

			describe('page 1', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangInBearbeitungAssignedPageLink(1)).as(loadPageInterceptor);
				})

				it('should show page 1 with 100 vorgaenge IN_BEARBEITUNG', () => {
					navigation.getMeineVorgaengeFilter().click();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 2', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangInBearbeitungAssignedPageLink(2)).as(loadPageInterceptor);
				})

				it('should show page 2 with 10 vorgaenge IN_BEARBEITUNG and 90 vorgaenge ASSIGNED on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === 10);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 101);
						assert(interception.response.body._embedded.vorgangHeaderList[9].name, MEINE_VORGAENGE_NAME_PREFIX + 110);
					})
				})
			})

			function buildVorgangInBearbeitungAssignedPageLink(page: number): string {
				return `${buildVorgangPageLink(page)}&assignedTo=${assignedTo}&status=IN_BEARBEITUNG`;
			}
		})

		describe('Nicht zugewiesen filter', () => {

			describe('page 1', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangInBearbeitungUnassignedPageLink(1)).as(loadPageInterceptor);
				})

				it('should show page 1 with 100 vorgaenge IN_BEARBEITUNG', () => {
					navigation.getNichtZugewiesenFilter().click();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, ALLE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, ALLE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 2', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangInBearbeitungUnassignedPageLink(2)).as(loadPageInterceptor);
				})

				it('should show page 2 with 10 vorgaenge on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === 10);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, ALLE_NAME_PREFIX + 101);
						assert(interception.response.body._embedded.vorgangHeaderList[9].name, ALLE_NAME_PREFIX + 110);
					})
				})
			})

			function buildVorgangInBearbeitungUnassignedPageLink(page: number): string {
				return `${buildVorgangPageLink(page)}&assignedTo=&status=IN_BEARBEITUNG`;
			}
		})
	})

	describe('Beschieden', () => {

		const ALLE_NAME_PREFIX: string = 'VorgangBESCHIEDEN_';
		const MEINE_VORGAENGE_NAME_PREFIX: string = 'VorgangBESCHIEDENAssigned_';

		describe('Alle filter', () => {

			describe('page 1', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					navigation.getAlleFilter().click();
					waitForSpinnerToDisappear();
					intercept(HttpMethodE2E.GET, buildVorgangBeschiedenPageLink(1)).as(loadPageInterceptor);
				})

				it('should show page 1 with 100 vorgaenge BESCHIEDEN', () => {
					views.getBeschieden().getRoot().click();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, ALLE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, ALLE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 2', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangBeschiedenPageLink(2)).as(loadPageInterceptor);
				})

				it('should show page 2 with 10 vorgaenge BESCHIEDEN and 90 vorgaenge ASSIGNED on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 3', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangBeschiedenPageLink(3)).as(loadPageInterceptor);
				})

				it('should show page 3 with 10 vorgaenge BESCHIEDEN ASSIGNED on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === 20);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 80);
						assert(interception.response.body._embedded.vorgangHeaderList[19].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			function buildVorgangBeschiedenPageLink(page: number): string {
				return `${buildVorgangPageLink(page)}&status=BESCHIEDEN`;
			}
		})

		describe('Meine Vorgänge filter', () => {

			describe('page 1', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangInBescheidenAssignedPageLink(1)).as(loadPageInterceptor);
				})

				it('should show page 1 with 100 vorgaenge BESCHIEDEN', () => {
					navigation.getMeineVorgaengeFilter().click();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 2', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangInBescheidenAssignedPageLink(2)).as(loadPageInterceptor);
				})

				it('should show page 2 with 10 vorgaenge BESCHIEDEN and 90 vorgaenge ASSIGNED on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === 10);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 101);
						assert(interception.response.body._embedded.vorgangHeaderList[9].name, MEINE_VORGAENGE_NAME_PREFIX + 110);
					})
				})
			})

			function buildVorgangInBescheidenAssignedPageLink(page: number): string {
				return `${buildVorgangPageLink(page)}&assignedTo=${assignedTo}&status=BESCHIEDEN`;
			}
		})

		describe('Nicht zugewiesen filter', () => {

			describe('page 1', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangInBescheidenUnassignedPageLink(1)).as(loadPageInterceptor);
				})

				it('should show page 1 with 100 vorgaenge BESCHIEDEN', () => {
					navigation.getNichtZugewiesenFilter().click();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, ALLE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, ALLE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 2', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangInBescheidenUnassignedPageLink(2)).as(loadPageInterceptor);
				})

				it('should show page 2 with 10 vorgaenge on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === 10);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, ALLE_NAME_PREFIX + 101);
						assert(interception.response.body._embedded.vorgangHeaderList[9].name, ALLE_NAME_PREFIX + 110);
					})
				})
			})

			function buildVorgangInBescheidenUnassignedPageLink(page: number): string {
				return `${buildVorgangPageLink(page)}&assignedTo=&status=BESCHIEDEN`;
			}
		})
	})

	describe('Abgeschlossen', () => {

		const ALLE_NAME_PREFIX: string = 'VorgangABGESCHLOSSEN_';
		const MEINE_VORGAENGE_NAME_PREFIX: string = 'VorgangABGESCHLOSSENAssigned_';

		describe('Alle filter', () => {

			describe('page 1', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					navigation.getAlleFilter().click();
					waitForSpinnerToDisappear();
					intercept(HttpMethodE2E.GET, buildVorgangAbgeschlossenPageLink(1)).as(loadPageInterceptor);
				})

				it('should show page 1 with 100 vorgaenge ABGESCHLOSSEN', () => {
					views.getAbgeschlossen().getRoot().click();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, ALLE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, ALLE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 2', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangAbgeschlossenPageLink(2)).as(loadPageInterceptor);
				})

				it('should show page 2 with 10 vorgaenge ABGESCHLOSSEN and 90 vorgaenge ASSIGNED on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 3', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangAbgeschlossenPageLink(3)).as(loadPageInterceptor);
				})

				it('should show page 3 with 10 vorgaenge ABGESCHLOSSEN ASSIGNED on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === 20);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 80);
						assert(interception.response.body._embedded.vorgangHeaderList[19].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			function buildVorgangAbgeschlossenPageLink(page: number): string {
				return `${buildVorgangPageLink(page)}&status=ABGESCHLOSSEN`;
			}
		})

		describe('Meine Vorgänge filter', () => {

			describe('page 1', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangInAbgeschlossenAssignedPageLink(1)).as(loadPageInterceptor);
				})

				it('should show page 1 with 100 vorgaenge ABGESCHLOSSEN', () => {
					navigation.getMeineVorgaengeFilter().click();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 2', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangInAbgeschlossenAssignedPageLink(2)).as(loadPageInterceptor);
				})

				it('should show page 2 with 10 vorgaenge ABGESCHLOSSEN and 90 vorgaenge ASSIGNED on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === 10);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 101);
						assert(interception.response.body._embedded.vorgangHeaderList[9].name, MEINE_VORGAENGE_NAME_PREFIX + 110);
					})
				})
			})

			function buildVorgangInAbgeschlossenAssignedPageLink(page: number): string {
				return `${buildVorgangPageLink(page)}&assignedTo=${assignedTo}&status=ABGESCHLOSSEN`;
			}
		})

		describe('Nicht zugewiesen filter', () => {

			describe('page 1', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangInAbgeschlossenUnassignedPageLink(1)).as(loadPageInterceptor);
				})

				it('should show page 1 with 100 vorgaenge ABGESCHLOSSEN', () => {
					navigation.getNichtZugewiesenFilter().click();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 2', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangInAbgeschlossenUnassignedPageLink(2)).as(loadPageInterceptor);
				})

				it('should show page 2 with 10 vorgaenge on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === 10);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 101);
						assert(interception.response.body._embedded.vorgangHeaderList[9].name, MEINE_VORGAENGE_NAME_PREFIX + 110);
					})
				})
			})

			function buildVorgangInAbgeschlossenUnassignedPageLink(page: number): string {
				return `${buildVorgangPageLink(page)}&assignedTo=&status=ABGESCHLOSSEN`;
			}
		})
	})

	describe('Verworfen', () => {

		const ALLE_NAME_PREFIX: string = 'VorgangVERWORFEN_';
		const MEINE_VORGAENGE_NAME_PREFIX: string = 'VorgangVERWORFENAssigned_';

		describe('Alle filter', () => {

			describe('page 1', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					navigation.getAlleFilter().click();
					waitForSpinnerToDisappear();
					intercept(HttpMethodE2E.GET, buildVorgangVerworfenPageLink(1)).as(loadPageInterceptor);
				})

				it('should show page 1 with 100 vorgaenge VERWORFEN', () => {
					views.getVerworfen().getRoot().click();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, ALLE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, ALLE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 2', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangVerworfenPageLink(2)).as(loadPageInterceptor);
				})

				it('should show page 2 with 10 vorgaenge VERWORFEN and 90 vorgaenge ASSIGNED on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 3', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangVerworfenPageLink(3)).as(loadPageInterceptor);
				})

				it('should show page 3 with 10 vorgaenge VERWORFEN ASSIGNED on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === 20);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 80);
						assert(interception.response.body._embedded.vorgangHeaderList[19].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			function buildVorgangVerworfenPageLink(page: number): string {
				return `${buildVorgangPageLink(page)}&status=VERWORFEN`;
			}
		})

		describe('Meine Vorgänge filter', () => {

			describe('page 1', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangInVerworfenAssignedPageLink(1)).as(loadPageInterceptor);
				})

				it('should show page 1 with 100 vorgaenge VERWORFEN', () => {
					navigation.getMeineVorgaengeFilter().click();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, MEINE_VORGAENGE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 2', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangInVerworfenAssignedPageLink(2)).as(loadPageInterceptor);
				})

				it('should show page 2 with 10 vorgaenge VERWORFEN and 90 vorgaenge ASSIGNED on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === 10);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, MEINE_VORGAENGE_NAME_PREFIX + 101);
						assert(interception.response.body._embedded.vorgangHeaderList[9].name, MEINE_VORGAENGE_NAME_PREFIX + 110);
					})
				})
			})

			function buildVorgangInVerworfenAssignedPageLink(page: number): string {
				return `${buildVorgangPageLink(page)}&assignedTo=${assignedTo}&status=VERWORFEN`;
			}
		})

		describe('Nicht zugewiesen', () => {

			describe('page 1', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangInVerworfenUnassignedPageLink(1)).as(loadPageInterceptor);
				})

				it('should show page 1 with 100 vorgaenge VERWORFEN', () => {
					navigation.getNichtZugewiesenFilter().click();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === PAGE_SIZE);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, ALLE_NAME_PREFIX + 1);
						assert(interception.response.body._embedded.vorgangHeaderList[99].name, ALLE_NAME_PREFIX + PAGE_SIZE);
					})
				})
			})

			describe('page 2', () => {

				const loadPageInterceptor: string = 'loadPage';

				beforeEach(() => {
					intercept(HttpMethodE2E.GET, buildVorgangInVerworfenUnassignedPageLink(2)).as(loadPageInterceptor);
				})

				it('should show page 2 with 10 vorgaenge on scrolling to bottom', () => {
					scrollToWindowBottom();

					waitOfInterceptor(loadPageInterceptor).then((interception: Interception) => {
						assert(interception.response.body._embedded.vorgangHeaderList.length === 10);
						assert(interception.response.body._embedded.vorgangHeaderList[0].name, ALLE_NAME_PREFIX + 101);
						assert(interception.response.body._embedded.vorgangHeaderList[9].name, ALLE_NAME_PREFIX + 110);
					})
				})
			})

			function buildVorgangInVerworfenUnassignedPageLink(page: number): string {
				return `${buildVorgangPageLink(page)}&assignedTo=&status=VERWORFEN`;
			}
		})
	})

	function createVorgaengeWithStatus(status: VorgangStatusE2E): VorgangE2E[] {
		let count = 1;
		return generateIds(110).map(randomId => {
			const vorgang: VorgangE2E = { ...buildVorgang(randomId, 'Vorgang' + status + '_' + count), status };
			count++;
			return vorgang;
		})
	}

	function createAssignedVorgaengeWithStatus(status: VorgangStatusE2E): VorgangE2E[] {
		let count = 1;
		return generateIds(110).map(randomId => {
			const vorgang: VorgangE2E = { ...buildVorgang(randomId, 'Vorgang' + status + 'Assigned_' + count), status, assignedTo };
			count++;
			return vorgang;
		})
	}

	function buildVorgangPageLink(page: number): string {
		return `vorgangs?page=${page - 1}&limit=100`;
	}
})