/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { EingangE2E, VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { dropCollections } from '../../../support/cypress-helper';
import { exist, notExist } from '../../../support/cypress.util';
import { ORGANISATIONSEINHEITEN_ID_FOR_KFINDER, ORGANISATIONSEINHEITEN_ID_FOR_KORDNER } from '../../../support/data.util';
import { loginAsKfinder, loginAsKordner, loginAsRichard, loginAsZonk } from '../../../support/user-util';
import { buildVorgang, createVorgang, initVorgaenge, objectIds } from '../../../support/vorgang-util';

describe('VorgangList', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const eingangForKFinder: EingangE2E = { ...createVorgang().eingangs[0], zustaendigeStelle: { organisationseinheitenId: ORGANISATIONSEINHEITEN_ID_FOR_KFINDER } };
	const vorgangForKFinder: VorgangE2E = { ...buildVorgang(objectIds[0], 'VorgangVisibleToKFinder'), eingangs: [eingangForKFinder] };

	const eingangForKOrdner: EingangE2E = { ...createVorgang().eingangs[0], zustaendigeStelle: { organisationseinheitenId: ORGANISATIONSEINHEITEN_ID_FOR_KORDNER } };
	const vorgangForKOrdner: VorgangE2E = { ...buildVorgang(objectIds[1], 'VorgangVisibleToKOrdner'), eingangs: [eingangForKOrdner] };

	before(() => {
		initVorgaenge([vorgangForKFinder, vorgangForKOrdner]);
	})

	after(() => {
		dropCollections();
	})

	describe('filtered/authorized', () => {

		describe('by user kfinder', () => {

			before(() => {
				loginAsKfinder();

				waitForSpinnerToDisappear();
				exist(vorgangList.getRoot());
			})

			it('should show authorized vorgaenge', () => {
				exist(vorgangList.getListItem(vorgangForKFinder.name).getRoot());
				notExist(vorgangList.getListItem(vorgangForKOrdner.name).getRoot());
			})
		})

		describe('by user kordner', () => {

			before(() => {
				loginAsKordner();

				waitForSpinnerToDisappear();
				exist(vorgangList.getRoot());
			})

			it('should show authorized vorgaenge', () => {
				exist(vorgangList.getListItem(vorgangForKOrdner.name).getRoot());
				notExist(vorgangList.getListItem(vorgangForKFinder.name).getRoot());
			})
		})

		describe('by user zonk', () => {

			const vorgang: VorgangE2E = createVorgang();

			before(() => {
				loginAsZonk();

				waitForSpinnerToDisappear();
				exist(vorgangList.getRoot());
			})

			it('should show authorized empty vorgaenge', () => {
				notExist(vorgangList.getListItem(vorgangForKOrdner.name).getRoot());
				notExist(vorgangList.getListItem(vorgangForKFinder.name).getRoot());
				notExist(vorgangList.getListItem(vorgang.name).getRoot());
			})
		})

		describe('by user richard', () => {

			before(() => {
				loginAsRichard();

				waitForSpinnerToDisappear();
			})

			it('should show hint message while have no roles', () => {
				notExist(vorgangList.getRoot());
				exist(vorgangList.getNoRoleMessage());
			})
		})
	})
})