/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { AttachmentContainerE2EComponent, AttachmentListE2EComponent } from 'apps/goofy-e2e/src/components/attachment/attachment.e2e.component';
import { WiedervorlageSubnavigationE2EComponent } from 'apps/goofy-e2e/src/components/wiedervorlage/wiedervorlage-subnavigation';
import { BinaryFileSnackbarMessageE2E } from 'apps/goofy-e2e/src/model/binary-file';
import { WiedervorlageE2E } from 'apps/goofy-e2e/src/model/wiedervorlage';
import { dropCollections, readFileFromDownloads, wait } from 'apps/goofy-e2e/src/support/cypress-helper';
import { initVorgangAttachedItem } from 'apps/goofy-e2e/src/support/vorgang-attached-item-util';
import { SnackBarE2EComponent } from '../../../components/ui/snackbar.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { WiedervorlageInVorgangE2EComponent } from '../../../components/wiedervorlage/wiedervorlage-in-vorgang.e2e.component';
import { WiedervorlageE2EComponent } from '../../../components/wiedervorlage/wiedervorlage-page.e2e.component';
import { VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { WiedervorlagePage } from '../../../page-objects/wiedervorlage.po';
import { containClass, contains, exist, haveLength, notContainClass, notExist } from '../../../support/cypress.util';
import { TEST_FILE_WITHOUT_CONTENT, TEST_FILE_WITH_CONTENT, TEST_FILE_WITH_CONTENT_46MB } from '../../../support/data.util';
import { uploadEmptyFile, uploadFile } from '../../../support/file-upload';
import { loginAsSabine } from '../../../support/user-util';
import { createVorgang, initVorgang, objectIds } from '../../../support/vorgang-util';
import { createWiedervorlageAttachedItem, createWiedervorlageItem } from '../../../support/wiedervorlage-util';

describe('Wiedervorlage attachments', () => {
	const mainPage: MainPage = new MainPage();

	const snackBar: SnackBarE2EComponent = mainPage.getSnackBar();

	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const vorgangPage: VorgangPage = new VorgangPage();
	const wiedervorlageContainerInVorgang = vorgangPage.getWiedervorlagenContainer();

	const wiedervorlagePage: WiedervorlagePage = new WiedervorlagePage();
	const wiedervorlageContainer: WiedervorlageE2EComponent = wiedervorlagePage.getWiedervorlageContainer();
	const attachmentContainer: AttachmentContainerE2EComponent = wiedervorlageContainer.getAttachmentContainer();
	const attachmentList: AttachmentListE2EComponent = attachmentContainer.getList();
	const subnavigation: WiedervorlageSubnavigationE2EComponent = wiedervorlagePage.getSubnavigation();

	const snackbar: SnackBarE2EComponent = mainPage.getSnackBar();

	const WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF: string = 'WiedervorlageWithAttachments';

	const vorgang: VorgangE2E = createVorgang();
	const wiedervorlage: WiedervorlageE2E = createWiedervorlageItem('Wiedervorlage');
	const wiedervorlageAttachedItem = createWiedervorlageAttachedItem(objectIds[0], vorgang._id.$oid);

	before(() => {
		initVorgang(vorgang);
		initVorgangAttachedItem([wiedervorlageAttachedItem]);

		loginAsSabine();

		waitForSpinnerToDisappear();
		exist(vorgangList.getRoot());
	})

	after(() => {
		dropCollections();
	})

	describe('Upload attachment', () => {

		it('should show vorgang detail page on click on list item', () => {
			vorgangList.getListItem(vorgang.name).getRoot().click();
			waitForSpinnerToDisappear();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		it('should show wiedervorlage page on click on create wiedervorlage button', () => {
			wiedervorlageContainerInVorgang.getCreateWiedervorlageButton().click();
			waitForSpinnerToDisappear();

			exist(wiedervorlagePage.getRoot());
		})

		it('should show empty attachment after upload', () => {
			uploadEmptyFile(attachmentContainer.getUploadInput(), TEST_FILE_WITHOUT_CONTENT);
			waitForSpinnerToDisappear();

			exist(attachmentList.getItem(TEST_FILE_WITHOUT_CONTENT).getRoot());
		})

		it('should download empty attachment on click', () => {
			attachmentList.getItem(TEST_FILE_WITHOUT_CONTENT).getDownloadButton().click();
			waitForSpinnerToDisappear();

			exist(readFileFromDownloads(TEST_FILE_WITHOUT_CONTENT));
		})

		it('should show attachment with content after download', () => {
			uploadFile(attachmentContainer.getUploadInput(), TEST_FILE_WITH_CONTENT);
			waitForSpinnerToDisappear();

			exist(attachmentList.getItem(TEST_FILE_WITH_CONTENT).getRoot());
		})

		it('should error snackbar after upload to large file', { defaultCommandTimeout: 30000 }, () => {
			uploadFile(attachmentContainer.getUploadInput(), TEST_FILE_WITH_CONTENT_46MB);
			waitForSpinnerToDisappear();

			exist(snackbar.getMessage());
			contains(snackbar.getMessage(), BinaryFileSnackbarMessageE2E.ATTACHMENT_NOT_ADDED.replace('{size}', '40MB'));
		})

		it('should close snackbar on close button', () => {
			snackbar.getCloseButton().click();

			notExist(snackbar.getMessage());
		})

		it('should download attachment on click', () => {
			attachmentList.getItem(TEST_FILE_WITH_CONTENT).getDownloadButton().click();
			waitForSpinnerToDisappear();

			exist(readFileFromDownloads(TEST_FILE_WITH_CONTENT));
		})

		it('should show snackbar after save wiedervorlage', () => {
			wiedervorlageContainer.getBetreff().clear().type(WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF);
			wiedervorlageContainer.getSpeichernButton().click();

			exist(snackbar.getMessage());
			contains(snackbar.getMessage(), 'Die Wiedervorlage "' + WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF + '" wurde angelegt');
			exist(snackbar.getCloseButton());
		})

		it('should close snackBar', () => {
			snackbar.getCloseButton().click();

			notExist(snackbar.getMessage());
		})

		it('should show attachments on wiedervorlage in vorgang-detail after save ', () => {
			wiedervorlageContainerInVorgang.getWiedervorlage(WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF).expandItem();

			exist(wiedervorlageContainerInVorgang.getWiedervorlage(WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF).getAttachmentContainer().getList().getItem(TEST_FILE_WITHOUT_CONTENT).getRoot());
			exist(wiedervorlageContainerInVorgang.getWiedervorlage(WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF).getAttachmentContainer().getList().getItem(TEST_FILE_WITH_CONTENT).getRoot());
		})

		it('should download empty attachment on click in list', () => {
			wiedervorlageContainerInVorgang.getWiedervorlage(wiedervorlage.betreff).getAttachmentContainer().getList().getItem(TEST_FILE_WITHOUT_CONTENT).getDownloadButton().click();
			waitForSpinnerToDisappear();

			exist(readFileFromDownloads(TEST_FILE_WITHOUT_CONTENT));
		})

		it('should download attachment on click in list', () => {
			wiedervorlageContainerInVorgang.getWiedervorlage(wiedervorlage.betreff).getAttachmentContainer().getList().getItem(TEST_FILE_WITH_CONTENT).getDownloadButton().click();
			waitForSpinnerToDisappear();

			exist(readFileFromDownloads(TEST_FILE_WITH_CONTENT));
		})
	})

	describe('Delete attachment', () => {

		const wiedervorlageComp: WiedervorlageInVorgangE2EComponent = wiedervorlageContainerInVorgang.getWiedervorlage(WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF);

		it('should open wiedervorlage page', () => {
			wiedervorlageComp.getLink().click();
			waitForSpinnerToDisappear();

			exist(attachmentList.getItem(TEST_FILE_WITHOUT_CONTENT).getRoot());
			exist(attachmentList.getItem(TEST_FILE_WITH_CONTENT).getRoot());
		})

		it('delete', () => {
			attachmentList.getItem(TEST_FILE_WITHOUT_CONTENT).getDeleteButton().click();
			waitForSpinnerToDisappear();

			notExist(attachmentList.getItem(TEST_FILE_WITHOUT_CONTENT).getRoot());
			exist(attachmentList.getItem(TEST_FILE_WITH_CONTENT).getRoot());
		})

		it('should show snackbar on save', () => {
			wiedervorlageContainer.getSpeichernButton().click();

			exist(snackbar.getMessage());
			contains(snackbar.getMessage(), `Die Wiedervorlage "${WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF}" wurde gespeichert`);
			snackbar.getCloseButton().click();
		})

		it('check attachments in wiedervorlage list', () => {
			waitForSpinnerToDisappear();

			wiedervorlageComp.getExpandButton();
			wiedervorlageComp.expandItem();

			notExist(wiedervorlageComp.getAttachmentContainer().getList().getItem(TEST_FILE_WITHOUT_CONTENT).getRoot());
			exist(wiedervorlageComp.getAttachmentContainer().getList().getItem(TEST_FILE_WITH_CONTENT).getRoot());
		})
	})

	describe('switch to wiedervorlage without attachments', () => {

		const wiedervorlageComp: WiedervorlageInVorgangE2EComponent = wiedervorlageContainerInVorgang.getWiedervorlage(wiedervorlage.betreff);

		it('should not show any attachments', () => {
			wiedervorlageComp.getLink().click();
			waitForSpinnerToDisappear();

			notExist(wiedervorlageComp.getAttachmentContainer().getList().getItem(TEST_FILE_WITHOUT_CONTENT).getRoot());
			notExist(wiedervorlageComp.getAttachmentContainer().getList().getItem(TEST_FILE_WITH_CONTENT).getRoot());

			wiedervorlagePage.getSubnavigation().navigateBack();
		})
	})

	describe('Same number of attachments after status change', () => {
		const wiedervorlageComp: WiedervorlageInVorgangE2EComponent = wiedervorlageContainerInVorgang.getWiedervorlage(WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF);

		it('should open wiedervorlage page', () => {
			wait(500);
			wiedervorlageComp.getLink().click();

			waitForSpinnerToDisappear();

			exist(attachmentList.getItem(TEST_FILE_WITH_CONTENT).getRoot());
		})

		it('should mark as erledigt', () => {
			subnavigation.erledigen();

			waitForSpinnerToDisappear();

			containClass(wiedervorlageContainer.getStatusDot(), 'erledigt');
			contains(snackBar.getMessage(), `Die Wiedervorlage ${WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF} wurde erledigt`);
		})

		it('should close snackBar on close', () => {
			snackBar.getCloseButton().click();

			notExist(snackBar.getMessage());
		})

		it('should mark as open', () => {
			subnavigation.wiedereroeffnen();

			waitForSpinnerToDisappear();

			notContainClass(wiedervorlageContainer.getStatusDot(), 'erledigt');
			contains(snackBar.getMessage(), `Die Wiedervorlage ${WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF} wurde wiedereröffnet`);
		})

		it('should close snackBar on close', () => {
			snackBar.getCloseButton().click();

			notExist(snackBar.getMessage());
		})

		it('should still have 1 attachment only', () => {
			const attachmentList: AttachmentListE2EComponent = wiedervorlageContainerInVorgang.getWiedervorlage(wiedervorlage.betreff).getAttachmentContainer().getList();

			haveLength(attachmentList.getRoot(), 1);
		})
	})

	describe('delete last attachment', () => {

		it('should remove on delete', () => {
			attachmentList.getItem(TEST_FILE_WITH_CONTENT).getDeleteButton().click();
			waitForSpinnerToDisappear();

			notExist(attachmentList.getItem(TEST_FILE_WITH_CONTENT).getRoot());
		})

		it('should show snackbar on save', () => {
			wiedervorlageContainer.getSpeichernButton().click();
			waitForSpinnerToDisappear();

			exist(snackbar.getMessage());
			contains(snackbar.getMessage(), `Die Wiedervorlage "${WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF}" wurde gespeichert`);
		})

		it('should close snackBar on close', () => {
			snackBar.getCloseButton().click();

			notExist(snackBar.getMessage());
		})

		it('should have no attachments after save', () => {
			const wiedervorlageComp: WiedervorlageInVorgangE2EComponent = wiedervorlageContainerInVorgang.getWiedervorlage(WIEDERVORLAGE_WITH_ATTACHMENTS_BETREFF);

			wiedervorlageComp.getExpandButton();
			wiedervorlageComp.expandItem();

			notExist(wiedervorlageComp.getAttachmentContainer().getList().getItem(TEST_FILE_WITHOUT_CONTENT).getRoot());
			notExist(wiedervorlageComp.getAttachmentContainer().getList().getItem(TEST_FILE_WITH_CONTENT).getRoot());
		})
	})
})