/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { UntypedFormBuilder, UntypedFormControl, UntypedFormGroup } from '@angular/forms';
import { ForwardingService } from '@goofy-client/forwarding-shared';
import { mock, useFromMock } from '@goofy-client/test-utils';
import { createVorgangResource } from 'libs/vorgang-shared/test/vorgang';
import { of } from 'rxjs';
import { VorgangForwardFormService } from './vorgang-forward.formservice';

describe('VorgangForwardFormService', () => {
	let formService: VorgangForwardFormService;
	let service;

	const formBuilder: UntypedFormBuilder = new UntypedFormBuilder();

	beforeEach(() => {
		service = mock(ForwardingService);

		formService = new VorgangForwardFormService(formBuilder, useFromMock(service));
	})

	it('should create', () => {
		expect(formService).toBeTruthy();
	});

	describe('submit', () => {

		const returnValue = {};

		beforeEach(() => {
			service.forward.mockReturnValue(of(returnValue));
			service.doSubmit = jest.fn();
		})

		it('should call service', () => {
			formService.vorgang = createVorgangResource();

			formService.submit();

			expect(service.forward).toHaveBeenCalledWith(formService.vorgang, formService.form.value);
		})
	})

	describe('patch field', () => {

		const controlName: string = 'testField';
		const value: string = 'testField';

		const form: UntypedFormGroup = <any>{
			controls: {
				[controlName]: new UntypedFormControl()
			}
		};

		it('should set control value', () => {
			formService.form = form;

			formService.patchField(controlName, value);

			expect(formService.form.controls[controlName].value).toEqual(value);
		})
	})
})