/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Component, Input } from '@angular/core';
import { CommandOrder, CommandResource } from '@goofy-client/command-shared';
import { PostfachMail } from '@goofy-client/postfach-shared';

@Component({
	selector: 'goofy-client-historie-item-postfach-nachricht',
	templateUrl: './historie-item-postfach-nachricht.component.html',
	styleUrls: ['./historie-item-postfach-nachricht.component.scss'],
})
export class HistorieItemPostfachNachrichtComponent {

	@Input() command: CommandResource;

	headline: string;
	postfachNachricht: PostfachMail;

	ngOnInit(): void {
		this.headline = HISTORIE_TEXT_BY_POSTFACH_NACHRICHT_ORDER[this.command.order];
		this.postfachNachricht = this.getPostfachNachricht();
	}

	getPostfachNachricht(): PostfachMail {
		if (this.command.order == CommandOrder.SEND_POSTFACH_NACHRICHT) {
			return this.command.body;
		}
		return this.command.body.item;
	}
}

const HISTORIE_TEXT_BY_POSTFACH_NACHRICHT_ORDER: { [order: string]: string } = {
	[CommandOrder.SEND_POSTFACH_NACHRICHT]: 'eine Nachricht geschrieben.',
	[CommandOrder.RECEIVE_POSTFACH_NACHRICHT]: 'eine Nachricht des Antragstellers empfangen.',
	[CommandOrder.RESEND_POSTFACH_NACHRICHT]: 'eine Nachricht erneut versendet.'
}