/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatExpansionModule } from '@angular/material/expansion';
import { CommandListResource } from '@goofy-client/command-shared';
import { createEmptyStateResource, createStateResource, FormatDateWithTimePipe, ToEmbeddedResourcesPipe } from '@goofy-client/tech-shared';
import { getElementFromFixture } from '@goofy-client/test-utils';
import { AccordionComponent, SpinnerComponent } from '@goofy-client/ui';
import { createCommandListResource } from 'libs/command-shared/test/command';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { HistorieItemVorgangCreatedComponent } from './historie-item-vorgang-created/historie-item-vorgang-created.component';
import { HistorieListItemComponent } from './historie-list-item/historie-list-item.component';
import { HistorieListComponent } from './historie-list.component';

registerLocaleData(localeDe);

describe('HistorieListComponent', () => {
	let component: HistorieListComponent;
	let fixture: ComponentFixture<HistorieListComponent>;

	const historieItem: string = getDataTestIdOf('historie-list-item-0');

	const commandListResource: CommandListResource = createCommandListResource();

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			imports: [
				MatExpansionModule,
			],
			declarations: [
				ToEmbeddedResourcesPipe,
				FormatDateWithTimePipe,
				HistorieListComponent,
				MockComponent(HistorieListItemComponent),
				MockComponent(SpinnerComponent),
				MockComponent(HistorieItemVorgangCreatedComponent),
				MockComponent(AccordionComponent)
			],
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(HistorieListComponent);
		component = fixture.componentInstance;
		component.commandStateListResource = createStateResource(commandListResource);
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('list item', () => {

		it('should show items on filled content', () => {
			component.commandStateListResource = createStateResource(commandListResource);
			fixture.detectChanges();

			const element = getElementFromFixture(fixture, historieItem);

			expect(element).toBeInstanceOf(HTMLElement);
		})

		it('should NOT show items without content', () => {
			component.commandStateListResource = createEmptyStateResource();
			fixture.detectChanges();

			const element = getElementFromFixture(fixture, historieItem);

			expect(element).not.toBeInstanceOf(HTMLElement);
		})
	})
})
