/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { mock, useFromMock } from '@goofy-client/test-utils';
import { VorgangHeaderLinkRel, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { ResourceFactory } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { createKommentarListResource } from '../../test/kommentar';
import { KommentarListLinkRel } from './kommentar.linkrel';
import { KommentarListResource } from './kommentar.model';
import { KommentarRepository } from './kommentar.repository';

describe('KommentarRepository', () => {
	let repository: KommentarRepository;
	let resourceFactory = mock(ResourceFactory);
	let resourceWrapper = { get: jest.fn() };

	beforeEach(() => {
		repository = new KommentarRepository(useFromMock(resourceFactory));
	})

	describe('findKommentare', () => {
		const kommentarListResource: KommentarListResource = createKommentarListResource();
		const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource([KommentarListLinkRel.KOMMENTAR_LIST]);

		beforeEach(() => {
			resourceFactory.from.mockReturnValue(resourceWrapper);
			resourceWrapper.get.mockReturnValue(cold('a', { a: kommentarListResource }));
		})

		it('should call resourceFactory with resource', () => {
			repository.findKommentare(vorgang);

			expect(resourceFactory.from).toHaveBeenCalledWith(vorgang);
		})

		it('should call resourceWrapper with link', () => {
			repository.findKommentare(vorgang);

			expect(resourceWrapper.get).toHaveBeenCalledWith(VorgangHeaderLinkRel.KOMMENTARE);
		})

		it('should return result', () => {
			let result = repository.findKommentare(vorgang);

			expect(result).not.toBeNull();
			expect(result).toBeObservable(hot('a', { a: kommentarListResource }));
		})
	})
})