/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule } from '@angular/forms';
import { MatFormFieldModule } from '@angular/material/form-field';
import { BinaryFileAttachmentContainerComponent } from '@goofy-client/binary-file';
import { KommentarLinkRel, KommentarService } from '@goofy-client/kommentar-shared';
import { createStateResource } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { OzgcloudStrokedButtonWithSpinnerComponent, TextAreaEditorComponent } from '@goofy-client/ui';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { createBinaryFileListResource } from '../../../../../binary-file-shared/test/binary-file';
import { createKommentarResource } from '../../../../../kommentar-shared/test/kommentar';
import { KommentarFormComponent } from './kommentar-form.component';
import { KommentarFormService } from './kommentar.formservice';

describe('KommentarFormComponent', () => {
	let component: KommentarFormComponent;
	let fixture: ComponentFixture<KommentarFormComponent>;

	const formService = mock(KommentarFormService);
	const kommentarService = mock(KommentarService);

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				KommentarFormComponent,
				MockComponent(TextAreaEditorComponent),
				MockComponent(OzgcloudStrokedButtonWithSpinnerComponent),
				MockComponent(BinaryFileAttachmentContainerComponent),
			],
			imports: [
				MatFormFieldModule,
				ReactiveFormsModule,
			],
			providers: [
				{
					provide: KommentarFormService,
					useValue: formService
				},
				{
					provide: KommentarService,
					useValue: kommentarService
				}
			]
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(KommentarFormComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	it('should call patch', () => {
		const patchSpy = jest.spyOn(KommentarFormService.prototype, 'patch').mockImplementation();
		const kommentarResource = createKommentarResource([KommentarLinkRel.ATTACHMENTS]);
		component.kommentar = kommentarResource;
		kommentarService.getAttachments.mockReturnValue(
			of(createStateResource(createBinaryFileListResource())));

		component.ngOnChanges();

		expect(patchSpy).toHaveBeenCalledWith(kommentarResource);
	})

	it('should load attachments', done => {
		component.kommentar = createKommentarResource([KommentarLinkRel.ATTACHMENTS]);
		kommentarService.getAttachments.mockReturnValue(
			of(createStateResource(createBinaryFileListResource())));

		component.ngOnChanges();

		component.attachments$.subscribe(binaryFileResources => {
			expect(binaryFileResources.length).toBe(10);
			done();
		});
	});

	it('should call kommentarService', () => {
		const kommentarResource = createKommentarResource([KommentarLinkRel.ATTACHMENTS]);
		component.kommentar = kommentarResource;
		kommentarService.getAttachments.mockReturnValue(
			of(createStateResource(createBinaryFileListResource())));

		component.ngOnChanges();

		expect(kommentarService.getAttachments).toHaveBeenCalledWith(kommentarResource);
	});
});
