/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Component, Input, OnInit } from '@angular/core';
import { BinaryFileListResource } from '@goofy-client/binary-file-shared';
import { KommentarLinkRel, KommentarListResource, KommentarResource, KommentarService } from '@goofy-client/kommentar-shared';
import { StateResource, createEmptyStateResource } from "@goofy-client/tech-shared";
import { hasLink } from '@ngxp/rest';
import { Observable, of } from 'rxjs';

@Component({
	selector: 'goofy-client-kommentar-list-item-in-vorgang',
	templateUrl: './kommentar-list-item-in-vorgang.component.html',
	styleUrls: ['./kommentar-list-item-in-vorgang.component.scss']
})
export class KommentarListItemInVorgangComponent implements OnInit {
	@Input() kommentar: KommentarResource;

	@Input() kommentarListStateResource: StateResource<KommentarListResource>;
	attachments$: Observable<StateResource<BinaryFileListResource>> = of(createEmptyStateResource<BinaryFileListResource>());

	editMode: boolean = false;

	readonly kommentarLinkRel = KommentarLinkRel;

	constructor(public kommentarService: KommentarService) {
	}

	ngOnInit(): void {
		this.attachments$ = this.kommentarService.getAttachments(this.kommentar);
	}

	edit(): void {
		this.editMode = hasLink(this.kommentar, KommentarLinkRel.EDIT);
	}
}