/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Component, Input } from '@angular/core';
import { MatDialogRef } from '@angular/material/dialog';
import { CommandResource } from '@goofy-client/command-shared';
import { PostfachMailFormDialogData, PostfachMailListLinkRel, PostfachMailListResource, PostfachService } from '@goofy-client/postfach-shared';
import { EMPTY_STRING, StateResource, hasError, isNotNull, isNotUndefined } from '@goofy-client/tech-shared';
import { DialogService, FixedDialogComponent } from '@goofy-client/ui';
import { VorgangHeaderLinkRel, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { hasLink } from '@ngxp/rest';
import { Observable } from 'rxjs';
import { tap } from 'rxjs/operators';
import { PostfachMailFormComponent } from '../postfach-mail-form/postfach-mail-form.component';

@Component({
	selector: 'goofy-client-postfach-mail-button-container',
	templateUrl: './postfach-mail-button-container.component.html',
	styleUrls: ['./postfach-mail-button-container.component.scss']
})
export class PostfachMailButtonContainerComponent {

	private _vorgang: VorgangWithEingangResource;
	public get vorgang(): VorgangWithEingangResource {
		return this._vorgang;
	}
	@Input()
	public set vorgang(value: VorgangWithEingangResource) {
		this._vorgang = value;
		if (isNotNull(value) && hasLink(value, VorgangHeaderLinkRel.POSTFACH_MAILS)) {
			this.loadPostfachMailListStateResource();
			this.loadPendingSendPostfachMailCommand();
		}
	}
	@Input() showAsIconButton: boolean;
	@Input() text: string;
	@Input() toolTip: string;

	pendingSendPostfachMailCommand$: Observable<StateResource<CommandResource>>;
	postfachMailListStateResource$: Observable<StateResource<PostfachMailListResource>>;

	dialogRef: MatDialogRef<FixedDialogComponent>;

	readonly postfachMailListLinkRel = PostfachMailListLinkRel;

	constructor(private dialogService: DialogService, private postfachService: PostfachService) { }

	loadPostfachMailListStateResource(): void {
		this.postfachMailListStateResource$ = this.postfachService.getPostfachMailListByGivenVorgang(this.vorgang);
	}

	loadPendingSendPostfachMailCommand(): void {
		this.pendingSendPostfachMailCommand$ = this.postfachService.getPendingSendPostfachMailCommand().pipe(
			tap(commandStateResource => this.closeDialog(commandStateResource))
		);
	}

	closeDialog(commandStateResource: StateResource<CommandResource>): void {
		if (!hasError(commandStateResource) && commandStateResource.loaded && isNotUndefined(this.dialogRef)) {
			this.dialogService.closeAll();
		}
	}

	public openPostfachMailDialog(postfachMailListStateResource: StateResource<PostfachMailListResource>): void {
		this.dialogRef = this.dialogService.openFixed(this.buildDialogData(postfachMailListStateResource));
	}

	private buildDialogData(postfachMailListStateResource: StateResource<PostfachMailListResource>): PostfachMailFormDialogData {
		return {
			component: PostfachMailFormComponent,
			title: 'Neue Nachricht',
			empfaenger: this.getEmpfaenger(),
			postfachMailListStateResource: postfachMailListStateResource
		}
	}

	getEmpfaenger(): string {
		return this.existsAntragsteller() ? `${this.vorgang.eingang.antragsteller.vorname} ${this.vorgang.eingang.antragsteller.nachname}` : EMPTY_STRING;
	}

	private existsAntragsteller(): boolean {
		return !!this.vorgang.eingang.antragsteller;
	}
}