import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormControl, FormGroup, FormGroupDirective, ReactiveFormsModule } from '@angular/forms';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { FORBIDDEN_REPLY_OPTION_ITEM, POSSIBLE_REPLY_OPTION_ITEM, PostfachService } from '@goofy-client/postfach-shared';
import { Mock, existsAsHtmlElement, getMockComponent, mock, notExistsAsHtmlElement } from '@goofy-client/test-utils';
import { EnumEditorComponent } from '@goofy-client/ui';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { PostfachMailFormservice } from '../postfach-mail.formservice';
import { PostfachNachrichtReplyEditorContainerComponent } from './postfach-nachricht-reply-editor-container.component';

describe('PostfachNachrichtReplyEditorContainerComponent', () => {
	let component: PostfachNachrichtReplyEditorContainerComponent;
	let fixture: ComponentFixture<PostfachNachrichtReplyEditorContainerComponent>;

	const replyOptionEditor: string = getDataTestIdOf('postfach-reply-option');

	const service: Mock<PostfachService> = mock(PostfachService);

	const formGroupDirective: FormGroupDirective = new FormGroupDirective([], []);
	const formGroup: FormGroup = new FormGroup({ [PostfachMailFormservice.FIELD_REPLY_OPTION]: new FormControl('') });

	beforeEach(async () => {
		formGroupDirective.form = formGroup;

		await TestBed.configureTestingModule({
			declarations: [
				PostfachNachrichtReplyEditorContainerComponent,
				MockComponent(EnumEditorComponent)
			],
			providers: [
				{
					provide: PostfachService,
					useValue: service
				},
				{
					provide: FormGroupDirective,
					useValue: formGroupDirective
				}
			],
			imports: [
				ReactiveFormsModule,
				BrowserAnimationsModule
			],
		}).compileComponents();

		fixture = TestBed.createComponent(PostfachNachrichtReplyEditorContainerComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('ngOnInit', () => {

		it('should get list stateResource', () => {
			component.ngOnInit();

			expect(service.getFeatures).toHaveBeenCalled();
		})
	})

	describe('replyOptionEditor', () => {

		describe('on available feature', () => {

			beforeEach(() => {
				component.postfachFeatures$ = of({ reply: true });
			})

			it('should be visible', () => {
				fixture.detectChanges();

				existsAsHtmlElement(fixture, replyOptionEditor);
			})

			it.skip('should be called with formControlName', () => {
				fixture.detectChanges();

				const enumEditor: EnumEditorComponent = getMockComponent(fixture, EnumEditorComponent);
				expect(enumEditor.control).toBe(PostfachMailFormservice.FIELD_REPLY_OPTION);
			})

			it('should be called with defaultItem', () => {
				fixture.detectChanges();

				const enumEditor: EnumEditorComponent = getMockComponent(fixture, EnumEditorComponent);
				expect(enumEditor.defaultItem).toBe(POSSIBLE_REPLY_OPTION_ITEM);
			})

			it('should be called wirh itemList', () => {
				fixture.detectChanges();

				const enumEditor: EnumEditorComponent = getMockComponent(fixture, EnumEditorComponent);
				expect(enumEditor.itemList).toEqual([POSSIBLE_REPLY_OPTION_ITEM, FORBIDDEN_REPLY_OPTION_ITEM]);
			})
		})

		describe('on unavailable feature', () => {

			beforeEach(() => {
				component.postfachFeatures$ = of({ reply: false });
			})

			it('should be hidden', () => {
				fixture.detectChanges();

				notExistsAsHtmlElement(fixture, replyOptionEditor);
			})
		})
	})
});

