/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { ON_PAGE, PostfachMailLinkRel } from '@goofy-client/postfach-shared';
import { FormatDateWithTimePipe, HasLinkPipe } from '@goofy-client/tech-shared';
import { getElementFromFixture } from '@goofy-client/test-utils';
import { MockComponent } from 'ng-mocks';
import { createPostfachMailResource } from '../../../../../../../postfach-shared/test/postfach';
import { PostfachMailDateComponent } from '../postfach-mail-date/postfach-mail-date.component';
import { IncommingMailComponent } from './incomming-mail.component';

registerLocaleData(localeDe);

describe('IncommingMailComponent', () => {
	let component: IncommingMailComponent;
	let fixture: ComponentFixture<IncommingMailComponent>;

	const attachmentIcon: string = '[data-test-id="postfach-nachricht-attachment-icon"]';

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				IncommingMailComponent,
				MatIcon,
				FormatDateWithTimePipe,
				MockComponent(PostfachMailDateComponent),
				HasLinkPipe,
				MatIcon,
			],
			providers: [
				{
					provide: ON_PAGE,
					useValue: undefined
				}
			]
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(IncommingMailComponent);
		component = fixture.componentInstance;
		component.postfachMail = createPostfachMailResource();
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('attachments', () => {

		it('should only show attachment icon on detail', () => {
			component.onPage = false;
			component.postfachMail = createPostfachMailResource([PostfachMailLinkRel.ATTACHMENTS]);
			fixture.detectChanges();

			const element = getElementFromFixture(fixture, attachmentIcon);

			expect(element).toBeInstanceOf(HTMLElement);
		})
	})
});
