/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIconModule } from '@angular/material/icon';
import { MAT_SNACK_BAR_DATA, MatSnackBarModule, MatSnackBarRef } from '@angular/material/snack-bar';
import { mock } from '@goofy-client/test-utils';
import { CommandLinkRel } from 'libs/command-shared/src/lib/command.linkrel';
import { createCommandResource } from 'libs/command-shared/test/command';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { SnackbarCloseButtonComponent } from '../snackbar-close-button/snackbar-close-button.component';
import { SnackBarData } from '../snackbar.model';
import { SnackbarInfoComponent } from './snackbar-info.component';

describe('SnackbarInfoComponent', () => {
	let component: SnackbarInfoComponent;
	let fixture: ComponentFixture<SnackbarInfoComponent>;

	const snackBarData: SnackBarData = { message: 'Message', commandResource: createCommandResource() };
	const snackBarRef = { ...mock(MatSnackBarRef), dissmis: jest.fn(), dismissWithAction: jest.fn() };

	const revokeButton: string = getDataTestIdOf('snackbar-revoke-button');

	beforeEach(async () => {
		TestBed.configureTestingModule({
			imports: [
				MatSnackBarModule,
				MatIconModule
			],
			declarations: [
				SnackbarInfoComponent,
				MockComponent(SnackbarCloseButtonComponent)
			],
			providers: [
				{
					provide: MAT_SNACK_BAR_DATA,
					useValue: { data: snackBarData }
				},
				{
					provide: MatSnackBarRef,
					useValue: snackBarRef
				}
			]
		})
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(SnackbarInfoComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('close button', () => {
		it('should close the snackbar', () => {
			component.close();

			expect(snackBarRef.dismiss).toHaveBeenCalled();
		})
	})

	describe('revoke button', () => {
		it('should be visible', () => {
			component.data = { message: 'Message', commandResource: createCommandResource([CommandLinkRel.REVOKE]) };

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(revokeButton);

			expect(button).toBeInstanceOf(HTMLElement);
		})

		it('should be hidden', () => {
			component.data = { message: 'Message', commandResource: createCommandResource() };

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(revokeButton);

			expect(button).not.toBeInstanceOf(HTMLElement);
		})

		it('should close the snackbar with action', () => {
			component.revoke();

			expect(snackBarRef.dismissWithAction).toHaveBeenCalled();
		})
	})
});
