/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { NoopAnimationsModule } from '@angular/platform-browser/animations';
import { createEmptyStateResource, createStateResource } from '@goofy-client/tech-shared';
import { getElementFromDomRoot, getElementFromFixture } from '@goofy-client/test-utils';
import { SpinnerComponent, UiModule } from '@goofy-client/ui';
import { UserIconComponent } from '@goofy-client/user-profile';
import { UserProfileResource, getUserName } from '@goofy-client/user-profile-shared';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createUserProfileResource } from 'libs/user-profile-shared/test/user-profile';
import { MockComponent } from 'ng-mocks';
import { UserProfileInHeaderComponent } from './user-profile-in-header.component';

describe('UserProfileInHeaderComponent', () => {
	let component: UserProfileInHeaderComponent;
	let fixture: ComponentFixture<UserProfileInHeaderComponent>;

	const logoutButton: string = getDataTestIdOf('logout-button');
	const userIconButton: string = getDataTestIdOf('user-icon-button');

	const userProfile: UserProfileResource = createUserProfileResource();

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				UserProfileInHeaderComponent,
				MatIcon,
				MockComponent(UserIconComponent),
				MockComponent(SpinnerComponent)
			],
			imports: [
				NoopAnimationsModule,
				UiModule
			]
		}).compileComponents();

		fixture = TestBed.createComponent(UserProfileInHeaderComponent);
		component = fixture.componentInstance;
		component.currentUserResource = createStateResource(userProfile);
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('click on logout button', () => {
		it('should emit logout event', () => {
			jest.spyOn(component.logoutEmitter, 'emit');

			getElementFromFixture(fixture, userIconButton).click();
			getElementFromDomRoot(fixture, logoutButton).click();

			expect(component.logoutEmitter.emit).toHaveBeenCalled();
		})
	})

	describe('button', () => {
		it('should have user name inside aria label', () => {
			const button: HTMLButtonElement = getElementFromFixture(fixture, userIconButton);
			const label: string = 'Benutzerkonto von ' + getUserName(component.currentUserResource.resource)

			expect(button).toHaveAttribute('aria-label', expect.stringContaining(label));
		})

		it('should not have user name inside aria label if no user loaded', () => {
			const button: HTMLButtonElement = getElementFromFixture(fixture, userIconButton);
			component.currentUserResource = createEmptyStateResource();

			fixture.detectChanges();

			expect(button).toHaveAttribute('aria-label', expect.stringContaining('Benutzerkonto'));
		})
	})
});
