/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Component, Input, OnInit } from '@angular/core';
import { CommandResource } from '@goofy-client/command-shared';
import { StateResource, createEmptyStateResource } from '@goofy-client/tech-shared';
import { VorgangCommandService, VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { Observable, of } from 'rxjs';

@Component({
	selector: 'goofy-client-annehmen-button',
	templateUrl: './annehmen-button.component.html',
	styleUrls: ['./annehmen-button.component.scss']
})
export class AnnehmenButtonComponent implements OnInit {

	@Input() vorgang: VorgangWithEingangResource;
	@Input() showAsIconButton: boolean = false;

	commandStateResource$: Observable<StateResource<CommandResource>> = of(createEmptyStateResource<CommandResource>());

	readonly linkRel = VorgangWithEingangLinkRel;

	constructor(private vorgangCommandService: VorgangCommandService) { }

	ngOnInit(): void {
		this.commandStateResource$ = this.vorgangCommandService.getAnnehmenCommand();
	}

	public annehmen(): void {
		this.vorgangCommandService.annehmen(this.vorgang);
	}
}