/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatDialogModule } from '@angular/material/dialog';
import { VorgangForwardingContainerComponent } from '@goofy-client/forwarding';
import { KommentarListInVorgangContainerComponent } from '@goofy-client/kommentar';
import { PostfachMailListContainerComponent } from '@goofy-client/postfach';
import { HasLinkPipe, ToEmbeddedResourcesPipe, createEmptyStateResource, createStateResource } from '@goofy-client/tech-shared';
import { OzgcloudStrokedButtonWithSpinnerComponent } from '@goofy-client/ui';
import { VorgangHeaderLinkRel, VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { WiedervorlageListInVorgangContainerComponent } from '@goofy-client/wiedervorlage';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { SpinnerComponent } from 'libs/ui/src/lib/ui/spinner/spinner.component';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { VorgangDetailAntragstellerComponent } from './vorgang-detail-antragsteller/vorgang-detail-antragsteller.component';
import { VorgangDetailAreaComponent } from './vorgang-detail-area.component';
import { VorgangDetailFormularButtonsComponent } from './vorgang-detail-formular-buttons/vorgang-detail-formular-buttons.component';
import { VorgangDetailFormularDatenComponent } from './vorgang-detail-formular-daten/vorgang-detail-formular-daten.component';
import { VorgangDetailHeaderComponent } from './vorgang-detail-header/vorgang-detail-header.component';

describe('VorgangDetailAreaComponent', () => {
	let component: VorgangDetailAreaComponent;
	let fixture: ComponentFixture<VorgangDetailAreaComponent>;

	const wiedervorlagenContainer: string = getDataTestIdOf('wiedervorlagen-container-in-vorgang');
	const kommentarContainer: string = getDataTestIdOf('kommentar-container-in-vorgang');
	const postfachNachrichtenContainer: string = getDataTestIdOf('postfach-nachrichten-container-in-vorgang');
	const forwardingContainer: string = getDataTestIdOf('forwarding-container-in-vorgang');

	const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();

	beforeEach(() => {
		TestBed.configureTestingModule({
			imports: [
				MatDialogModule
			],
			declarations: [
				VorgangDetailAreaComponent,
				HasLinkPipe,
				ToEmbeddedResourcesPipe,
				MockComponent(SpinnerComponent),
				MockComponent(VorgangDetailHeaderComponent),
				MockComponent(VorgangDetailFormularButtonsComponent),
				MockComponent(OzgcloudStrokedButtonWithSpinnerComponent),
				MockComponent(WiedervorlageListInVorgangContainerComponent),
				MockComponent(KommentarListInVorgangContainerComponent),
				MockComponent(PostfachMailListContainerComponent),
				MockComponent(VorgangDetailAntragstellerComponent),
				MockComponent(VorgangDetailFormularDatenComponent),
				MockComponent(VorgangForwardingContainerComponent),
			],
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangDetailAreaComponent);
		component = fixture.componentInstance;
		component.vorgangStateResource = createEmptyStateResource();
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('wiedervorlagen', () => {

		it('should be visible', () => {
			component.vorgangStateResource = createStateResource(createVorgangWithEingangResource([VorgangHeaderLinkRel.WIEDERVORLAGEN]));
			fixture.detectChanges();

			const element = fixture.nativeElement.querySelector(wiedervorlagenContainer);

			expect(element).toBeInstanceOf(HTMLElement);
		})

		it('should be hidden', () => {
			component.vorgangStateResource = createStateResource(vorgang);
			fixture.detectChanges();

			const element = fixture.nativeElement.querySelector(wiedervorlagenContainer);

			expect(element).not.toBeInstanceOf(HTMLElement);
		})
	})

	describe('kommentare', () => {

		it('should be visible', () => {
			component.vorgangStateResource = createStateResource(createVorgangWithEingangResource([VorgangHeaderLinkRel.KOMMENTARE]));
			fixture.detectChanges();

			const element = fixture.nativeElement.querySelector(kommentarContainer);

			expect(element).toBeInstanceOf(HTMLElement);
		})

		it('should be hidden', () => {
			component.vorgangStateResource = createStateResource(vorgang);
			fixture.detectChanges();

			const element = fixture.nativeElement.querySelector(kommentarContainer);

			expect(element).not.toBeInstanceOf(HTMLElement);
		})
	})

	describe('postfach nachrichten', () => {

		it('should be visible', () => {
			component.vorgangStateResource = createStateResource(createVorgangWithEingangResource([VorgangHeaderLinkRel.POSTFACH_MAILS]));
			fixture.detectChanges();

			const element = fixture.nativeElement.querySelector(postfachNachrichtenContainer);

			expect(element).toBeInstanceOf(HTMLElement);
		})

		it('should be hidden', () => {
			component.vorgangStateResource = createStateResource(vorgang);
			fixture.detectChanges();

			const element = fixture.nativeElement.querySelector(postfachNachrichtenContainer);

			expect(element).not.toBeInstanceOf(HTMLElement);
		})
	})

	describe('forwarding', () => {

		it('should be visible', () => {
			component.vorgangStateResource = createStateResource(createVorgangWithEingangResource([VorgangWithEingangLinkRel.FORWARDING]));
			fixture.detectChanges();

			const element = fixture.nativeElement.querySelector(forwardingContainer);

			expect(element).toBeInstanceOf(HTMLElement);
		})

		it('should be hidden', () => {
			component.vorgangStateResource = createStateResource(vorgang);
			fixture.detectChanges();

			const element = fixture.nativeElement.querySelector(forwardingContainer);

			expect(element).not.toBeInstanceOf(HTMLElement);
		})
	})
});