import { ChangeDetectorRef, SimpleChanges } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { NoopAnimationsModule } from '@angular/platform-browser/animations';
import { MenuItemComponent, OzgcloudIconComponent, UiModule } from '@goofy-client/ui';
import { MockComponent } from 'ng-mocks';
import { VorgangDetailMoreMenuComponent } from './vorgang-detail-more-menu.component';

describe('VorgangDetailMoreMenuComponent', () => {
	let component: VorgangDetailMoreMenuComponent;
	let fixture: ComponentFixture<VorgangDetailMoreMenuComponent>;

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				VorgangDetailMoreMenuComponent,
				MockComponent(MenuItemComponent),
				MockComponent(OzgcloudIconComponent),
			],
			imports: [
				UiModule,
				NoopAnimationsModule,
			]
		}).compileComponents();

		fixture = TestBed.createComponent(VorgangDetailMoreMenuComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('ngOnChanges', () => {

		it('should call vorgangWithEingangChanged if vorgangWithEingang changed', () => {
			const simpleChanges: SimpleChanges = <any>{ vorgangWithEingang: {} };
			const spy = jest.spyOn(component, 'vorgangWithEingangChanged');

			component.ngOnChanges(simpleChanges);

			expect(spy).toHaveBeenCalled();
		})

		it('should not call vorgangWithEingangChanged if vorgangWithEingang did not change', () => {
			const simpleChanges: SimpleChanges = <any>{ dummy: {} };
			const spy = jest.spyOn(component, 'vorgangWithEingangChanged');

			component.ngOnChanges(simpleChanges);

			expect(spy).not.toHaveBeenCalled();
		})
	})

	describe('vorgangWithEingangChanged', () => {
		it('should call detectChanges', () => {
			const changeDetectorRef = fixture.debugElement.injector.get(ChangeDetectorRef);
			const detectChangesSpy = jest.spyOn(changeDetectorRef.constructor.prototype, 'detectChanges');

			component.vorgangWithEingangChanged();

			expect(detectChangesSpy).toHaveBeenCalled();
		})

		it('should call isButtonDisabled', () => {
			const spy = jest.spyOn(component, 'isButtonEnabled');

			component.vorgangWithEingangChanged();

			expect(spy).toHaveBeenCalled();
		})
	})

	describe('isButtonEnabled', () => {
		it('should return true if at least 1 menu item exists', () => {
			component.menuItems.reset([new MenuItemComponent()]);

			const res: boolean = component.isButtonEnabled();

			expect(res).toBeTruthy();
		})

		it('should return false if no menu item exists', () => {
			component.menuItems.reset([]);

			const res: boolean = component.isButtonEnabled();

			expect(res).toBeFalsy();
		})
	})

	describe('closeMenu', () => {

		//TODO Auf das event vom Button ausweiten
		it.skip('should closeMenu by trigger', () => {
			jest.spyOn(component.trigger, 'closeMenu');

			component.closeMenu();

			expect(component.trigger.closeMenu).toHaveBeenCalled();
		})
	})
});
