import { ChangeDetectorRef, Component, Input, OnChanges, QueryList, SimpleChanges, ViewChild, ViewChildren } from '@angular/core';
import { MatMenuTrigger } from '@angular/material/menu';
import { isNotUndefined } from '@goofy-client/tech-shared';
import { MenuItemComponent } from '@goofy-client/ui';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { has } from 'lodash-es';

@Component({
	selector: 'goofy-client-vorgang-detail-more-menu',
	templateUrl: './vorgang-detail-more-menu.component.html',
	styleUrls: ['./vorgang-detail-more-menu.component.scss'],
})
export class VorgangDetailMoreMenuComponent implements OnChanges {

	@Input() vorgangWithEingang: VorgangWithEingangResource;

	@ViewChild('menuTrigger') trigger: MatMenuTrigger;
	@ViewChildren(MenuItemComponent) menuItems!: QueryList<MenuItemComponent>;

	buttonEnabled: boolean = false;

	readonly vorgangWithEingangLinkRel = VorgangWithEingangLinkRel;

	constructor(private changeDetectorRef: ChangeDetectorRef) { }

	ngOnChanges(changes: SimpleChanges): void {
		if (has(changes, 'vorgangWithEingang')) {
			this.vorgangWithEingangChanged();
		}
	}

	public vorgangWithEingangChanged(): void {
		this.changeDetectorRef.detectChanges();
		this.buttonEnabled = this.isButtonEnabled();
	}

	public isButtonEnabled(): boolean {
		return this.menuItems?.length > 0;
	}

	public closeMenu(): void {
		if(isNotUndefined(this.trigger)){
			this.trigger.closeMenu();
		}
	}
}
