/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CommandOrder, CreateCommand } from '@goofy-client/command-shared';
import { EMPTY_ARRAY, isNotNil } from '@goofy-client/tech-shared';
import { ResourceUri, getEmbeddedResource } from '@ngxp/rest';
import { isNull } from 'lodash-es';
import { VorgangListLinkRel } from './vorgang.linkrel';
import { CreateAssignUserCommand, CreateForwardCommand, ForwardRequest, VorgangListResource, VorgangOrder, VorgangResource } from './vorgang.model';

export function createZurueckholenCommand(): CreateCommand {
	return { order: VorgangOrder.ZURUECKHOLEN, body: null };
}

export function createAnnehmenCommand(): CreateCommand {
	return { order: VorgangOrder.ANNEHMEN, body: null };
}

export function createVerwerfenCommand(): CreateCommand {
	return { order: VorgangOrder.VERWERFEN, body: null };
}

export function createBearbeitenCommand(): CreateCommand {
	return { order: VorgangOrder.BEARBEITEN, body: null };
}

export function createBescheidenCommand(): CreateCommand {
	return { order: VorgangOrder.BESCHEIDEN, body: null };
}

export function createZurueckstellenCommand(): CreateCommand {
	return { order: VorgangOrder.ZURUECKSTELLEN, body: null };
}

export function createAbschliessenCommand(): CreateCommand {
	return { order: VorgangOrder.ABSCHLIESSEN, body: null };
}

export function createWiedereroeffnenCommand(): CreateCommand {
	return { order: VorgangOrder.WIEDEREROEFFNEN, body: null };
}

export function createForwardCommand(redirectRequest: ForwardRequest): CreateForwardCommand {
	return { order: VorgangOrder.FORWARD, redirectRequest, body: null };
}

export function getVorgaengeFromList(vorgangList: VorgangListResource): VorgangResource[] {
	if (isNotNil(vorgangList)) {
		const embeddedResource: VorgangResource[] = getEmbeddedResource(vorgangList, VorgangListLinkRel.VORGANG_HEADER_LIST);
		return isNull(embeddedResource) ? EMPTY_ARRAY : embeddedResource;
	}
	return EMPTY_ARRAY;
}

export function createAssignUserCommand(assignedTo: ResourceUri): CreateAssignUserCommand {
	return { order: CommandOrder.ASSIGN_USER, body: { assignedTo } };
}

export function isStatusCommand(order: CommandOrder): boolean {
	return [
		CommandOrder.VORGANG_ANNEHMEN,
		CommandOrder.VORGANG_BEARBEITEN,
		CommandOrder.VORGANG_BESCHEIDEN,
		CommandOrder.VORGANG_ABSCHLIESSEN,
		CommandOrder.VORGANG_VERWERFEN,
		CommandOrder.VORGANG_WIEDEREROEFFNEN,
		CommandOrder.VORGANG_ZURUECKHOLEN,
		CommandOrder.VORGANG_ZURUECKSTELLEN,
		CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN
	].includes(order);
}

export function isAssignUserCommand(order: CommandOrder): boolean {
	return order === CommandOrder.ASSIGN_USER;
}