/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Component, EventEmitter, HostListener, Input, Output } from '@angular/core';
import { ApiRootLinkRel, ApiRootResource } from '@goofy-client/api-root-shared';
import { StateResource, createEmptyStateResource } from '@goofy-client/tech-shared';
import { VorgangListLinkRel, VorgangListResource, VorgangResource } from '@goofy-client/vorgang-shared';
import { hasLink } from '@ngxp/rest';

@Component({
	selector: 'goofy-client-vorgang-list',
	templateUrl: './vorgang-list.component.html',
	styleUrls: ['./vorgang-list.component.scss']
})
export class VorgangListComponent {

	@Input() apiRootStateResource: StateResource<ApiRootResource> = createEmptyStateResource();

	@Input() vorgangListPageResource: StateResource<VorgangListResource>;
	@Input() vorgaenge: VorgangResource[];
	@Input() searchString: string;

	@Output() nextPage: EventEmitter<void> = new EventEmitter<void>();

	readonly apiRootLinkRel = ApiRootLinkRel;

	@HostListener('window:scroll', ['$event'])
	onWindowScroll(event) {//TODO event typisieren?!
		if (this.isScrolledToBottom(event)) {
			this.loadNextPage();
		}
	}

	isScrolledToBottom(event): boolean {
		const top = event.currentTarget.scrollY;
		const height = document.documentElement.scrollHeight;
		const innerHeight = event.currentTarget.innerHeight + 1;
		return top >= height - innerHeight;
	}

	loadNextPage(): void {
		if (this.shouldLoadNextPage()) {
			this.nextPage.emit();
		}
	}

	shouldLoadNextPage(): boolean {
		return this.isVorgangListNotLoading() && this.hasNextPage();
	}

	private isVorgangListNotLoading(): boolean {
		return this.vorgangListPageResource.loaded && !this.vorgangListPageResource.loading && !this.vorgangListPageResource.reload;
	}

	hasNextPage(): boolean {
		return hasLink(this.vorgangListPageResource.resource, VorgangListLinkRel.NEXT);
	}

	public isEmptySearchResult(): boolean {
		return this.isValidList() && this.notExistsVorgaenge();
	}

	private isValidList(): boolean {
		return !this.vorgangListPageResource.loading && this.vorgangListPageResource.loaded && !this.vorgangListPageResource.reload && !this.vorgangListPageResource.hasOwnProperty('error');
	}

	private notExistsVorgaenge(): boolean {
		return this.vorgaenge.length === 0;
	}
}
