/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ScrollingModule } from '@angular/cdk/scrolling';
import { CommonModule } from '@angular/common';
import { NgModule } from '@angular/core';
import { MatButtonToggleModule } from '@angular/material/button-toggle';
import { RouterModule, Routes } from '@angular/router';
import { HintModule } from '@goofy-client/hint';
import { TechSharedModule } from '@goofy-client/tech-shared';
import { UiModule } from '@goofy-client/ui';
import { UserProfileModule } from '@goofy-client/user-profile';
import { VorgangSharedModule } from '@goofy-client/vorgang-shared';
import { VorgangSharedUiModule } from '@goofy-client/vorgang-shared-ui';
import { WiedervorlageModule } from '@goofy-client/wiedervorlage';
import { vorgangFilterViewGuard } from './vorgang-filter-view.guard';
import { VorgangListContainerComponent } from './vorgang-list-container/vorgang-list-container.component';
import { EmptyListComponent } from './vorgang-list-container/vorgang-list/empty-list/empty-list.component';
import { VorgangCreatedAtComponent } from './vorgang-list-container/vorgang-list/vorgang-list-item/vorgang-created-at/vorgang-created-at.component';
import { VorgangListItemComponent } from './vorgang-list-container/vorgang-list/vorgang-list-item/vorgang-list-item.component';
import { VorgangNextFristButton } from './vorgang-list-container/vorgang-list/vorgang-list-item/vorgang-next-frist-button/vorgang-next-frist-button.component';
import { VorgangListComponent } from './vorgang-list-container/vorgang-list/vorgang-list.component';
import { VorgangListPageContainerComponent } from './vorgang-list-page-container/vorgang-list-page-container.component';
import { VorgangFilterMenuContainerComponent } from './vorgang-list-page-container/vorgang-list-page/vorgang-filter-menu-container/vorgang-filter-menu-container.component';
import { VorgangFilterMenuItemComponent } from './vorgang-list-page-container/vorgang-list-page/vorgang-filter-menu-container/vorgang-filter-menu/vorgang-filter-menu-item/vorgang-filter-menu-item.component';
import { VorgangFilterMenuComponent } from './vorgang-list-page-container/vorgang-list-page/vorgang-filter-menu-container/vorgang-filter-menu/vorgang-filter-menu.component';
import { VorgangListPageComponent } from './vorgang-list-page-container/vorgang-list-page/vorgang-list-page.component';
import { VorgangViewsMenuContainerComponent } from './vorgang-list-page-container/vorgang-list-page/vorgang-views-menu-container/vorgang-views-menu-container.component';
import { VorgangSearchViewItemContainerComponent } from './vorgang-list-page-container/vorgang-list-page/vorgang-views-menu/vorgang-search-view-item-container/vorgang-search-view-item-container.component';
import { VorgangSearchViewItemComponent } from './vorgang-list-page-container/vorgang-list-page/vorgang-views-menu/vorgang-search-view-item-container/vorgang-search-view-item/vorgang-search-view-item.component';
import { VorgangViewItemContainerComponent } from './vorgang-list-page-container/vorgang-list-page/vorgang-views-menu/vorgang-view-item-container/vorgang-view-item-container.component';
import { VorgangViewItemComponent } from './vorgang-list-page-container/vorgang-list-page/vorgang-views-menu/vorgang-view-item-container/vorgang-view-item/vorgang-view-item.component';
import { VorgangViewsMenuComponent } from './vorgang-list-page-container/vorgang-list-page/vorgang-views-menu/vorgang-views-menu.component';
import { vorgangListPageGuard } from './vorgang-list-page.guard';
import { VorgangListSearchContainerComponent } from './vorgang-list-search-container/vorgang-list-search-container.component';
import { VorgangListSearchComponent } from './vorgang-list-search-container/vorgang-list-search/vorgang-list-search.component';

const routes: Routes = [
	{
		path: '',
		component: VorgangListPageContainerComponent,
		title: 'Vorgänge | Alfa',
		children: [
			{
				path: '',
				title: 'Alle Vorgänge | Alfa',
				pathMatch: 'full',
				canActivate: [vorgangListPageGuard],
				children: [],
			},
			{
				path: 'alle',
				title: 'Alle Vorgänge | Alfa',
				component: VorgangListContainerComponent,
			},
			{
				path: 'alle/search/:search',
				title: 'Suchergebnisseite | Alfa',
				canActivate: [vorgangFilterViewGuard],
				component: VorgangListContainerComponent,
			},
			{
				path: 'alle/search',
				canActivate: [vorgangFilterViewGuard],
				component: VorgangListSearchContainerComponent,
			},
			{
				path: 'alle/:status',
				title: 'Alle Vorgänge | Alfa',
				canActivate: [vorgangFilterViewGuard],
				component: VorgangListContainerComponent,
			},
			{
				path: 'alle/wiedervorlagen',
				title: 'Alle Vorgänge mit offenen Wiedervorlagen',
				canActivate: [vorgangFilterViewGuard],
				component: VorgangListContainerComponent,
			},
			{
				path: 'meine',
				title: 'Meine Vorgänge | Alfa',
				canActivate: [vorgangFilterViewGuard],
				component: VorgangListContainerComponent,
			},
			{
				path: 'meine/search/:search',
				title: 'Suchergebnis in meine Vorgänge | Alfa',
				canActivate: [vorgangFilterViewGuard],
				component: VorgangListContainerComponent,
			},
			{
				path: 'meine/search',
				canActivate: [vorgangFilterViewGuard],
				component: VorgangListSearchContainerComponent,
			},
			{
				path: 'meine/:status',
				title: 'Meine Vorgänge | Alfa',
				canActivate: [vorgangFilterViewGuard],
				component: VorgangListContainerComponent,
			},
			{
				path: 'meine/wiedervorlagen',
				title: 'Alle mir zugewiesenen Vorgänge mit offenen Wiedervorlagen',
				canActivate: [vorgangFilterViewGuard],
				component: VorgangListContainerComponent,
			},
			{
				path: 'unassigned',
				title: 'Nicht zugewiesen | Alfa',
				canActivate: [vorgangFilterViewGuard],
				component: VorgangListContainerComponent,
			},
			{
				path: 'unassigned/search/:search',
				title: 'Suchergebnis in nicht zugewiesen | Alfa',
				canActivate: [vorgangFilterViewGuard],
				component: VorgangListContainerComponent,
			},
			{
				path: 'unassigned/search',
				canActivate: [vorgangFilterViewGuard],
				component: VorgangListSearchContainerComponent,
			},
			{
				path: 'unassigned/:status',
				title: 'Nicht zugewiesen | Alfa',
				canActivate: [vorgangFilterViewGuard],
				component: VorgangListContainerComponent,
			},
			{
				path: 'unassigned/wiedervorlagen',
				title: 'Alle nicht zugewiesenen Vorgänge mit offenen Wiedervorlagen',
				canActivate: [vorgangFilterViewGuard],
				component: VorgangListContainerComponent,
			},
		],
	},
	{
		path: 'vorgang/:vorgangWithEingangUrl',
		loadChildren: () =>
			import('@goofy-client/vorgang-detail').then(
				(m) => m.VorgangDetailModule
			),
		title: 'Details zum Vorgang | Alfa',
	},
];

@NgModule({
	imports: [
		CommonModule,
		TechSharedModule,
		ScrollingModule,
		UiModule,
		RouterModule.forChild(routes),
		VorgangSharedModule,
		WiedervorlageModule,
		VorgangSharedUiModule,
		UserProfileModule,
		MatButtonToggleModule,
		HintModule,
	],
	declarations: [
		VorgangListComponent,
		VorgangListItemComponent,
		VorgangListContainerComponent,
		VorgangListPageComponent,
		EmptyListComponent,
		VorgangNextFristButton,
		VorgangListPageContainerComponent,
		VorgangFilterMenuContainerComponent,
		VorgangFilterMenuComponent,
		VorgangViewsMenuComponent,
		VorgangSearchViewItemComponent,
		VorgangSearchViewItemContainerComponent,
		VorgangListSearchContainerComponent,
		VorgangFilterMenuItemComponent,
		VorgangCreatedAtComponent,
		VorgangViewItemContainerComponent,
		VorgangViewItemComponent,
		VorgangViewsMenuContainerComponent,
		VorgangListSearchComponent,
	],
})
export class VorgangModule {}
